// BABEL.C - Converting to CNT-format
// Written by Stefan Pfeiffer, DL1ELY, 951111
// Last modification: 960603
// (C) 1995 by Stefan Pfeiffer, DL1ELY

#include <stdio.h>
#include <string.h>
#include <mem.h>
#include <alloc.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include "locator.h"

#define NAME    "Babel fish"
#define VERSION "v1.10"
#define DATE    "960604"
#define AUTHOR  "Stefan Pfeiffer, DL1ELY"


#define QW      1
#define KLOG    2
#define UKWTEST 3


typedef struct header
{
  unsigned char magic[5];
  unsigned char owncall[14];
  unsigned char ownloc[7];
  unsigned char desc[60];
  unsigned qso_cnt;
  unsigned invalid;
  unsigned ODX;
  unsigned char ODX_call[14];
  unsigned average;
  unsigned char countries;
  unsigned char unknown_pref;
  unsigned char squares;
  unsigned long points;
};

typedef struct qso
{
  struct qso *next, *prev;

  unsigned hour, min;
  unsigned nr;
  unsigned char barecall[13];
  unsigned char call[14];
  unsigned char locator[7];
  unsigned char rst_RX[8];
  unsigned char rst_TX[4];
  unsigned points;
  unsigned direction;
};


typedef struct callsign
{
   struct callsign *next;

   unsigned char call[14];
};


// Splits a given call in its parts
void splitcall(char *rawcall,char *prefix,char *call,char *add)
{
   unsigned char hlp[81];
   char *cp1;
   char *cp2;

   *prefix = 0;
   *call = 0;
   *add = 0;

   cp1 = rawcall;
   cp2 = hlp;
   while(*cp1 && *cp1 != '/')
   {
      *cp2 = *cp1;
      cp1++;
      cp2++;
   }
   *cp2 = 0;

   if(!hlp[0]) return;

   if(strlen(hlp) > 3)
   {
      cp1 = rawcall;
   }
   else
   {
      strcpy(prefix,hlp);
      if(*cp1) cp1++;
   }

   cp2 = hlp;
   while(*cp1 && *cp1 != '/')
   {
      *cp2 = *cp1;
      cp1++;
      cp2++;
   }
   *cp2 = 0;

   if((strlen(hlp) < 4) || (strlen(hlp) > 12)) return;
   strcpy(call,hlp);

   if(!*cp1)
   {
      *add = 0;
   }
   else
   {
      cp1++;
      if(strlen(cp1) > 2)
      {
         *call = 0;
         return;
      }
      strcpy(add,cp1);
   }
};



void main(int argc, char **argv)
{
   FILE *f, *outf;
   unsigned char buf[1240];
   char slen;
   char s[20];
   struct header h;
   struct qso q;
   long rst_tx;
   long rst_rx;

   char squares[150][5];
   unsigned char i;

   char valid;

   double own_north, own_east;
   double north, east;
   double dist, dir;

   unsigned read;

   char pref[6];
   char add[4];

   char s_name[128];
   char d_name[128];
   char des_name[128];
   char *cp;

   char typ, ok;

   long rst;
   struct tm *t;

   struct callsign *first, *hlp;

   char band;
   char sel_band;
   char bands[256];
   unsigned last_nr;

   printf("\n%s %s,%s - Freeware by %s\n",NAME,VERSION,DATE,AUTHOR);

   if(argc<2)
   {
      printf("\nDieses Programm dient zur Umwandlung von Datendateien anderer Kontest-\n");
      printf("programme in das CNT2-Format von FiveNine. Es erfolgt eine bernahme der Log-\n");
      printf("daten und eine Neuberechnung der Punkte. Dabei kann es zu Differenzen mit den\n");
      printf("Punktzahlen anderer Programme kommen.\n");
      printf("\nMomentan werden die Formate folgender Programme untersttzt:\n");
      printf("  QW (Nur Region-1-Test!), K-Log, UKWTEST\n\n");
      printf("ACHTUNG:\n  Es lassen sich nur Files konvertieren, die aus einem einzelnen Band\n");
      printf("  bestehen, ansonsten kommt es zu falschen Ergebnissen.\n");
      printf("\nAufruf:\n");
      printf("  BABEL <Dateiname_mit_Extension>\n\n");
      printf("Zur Erklrung des Programmnamens empfehle ich die Lektre von \"Hitchhiker's\n");
      printf("Guide through the Galaxis\".\n\n");

      return;
   };

   strcpy(s_name,argv[1]);
   cp = s_name+strlen(s_name)-1;
   while((cp > s_name) && (*cp!='.')) cp--;

   if(cp > s_name)
   {
      *cp = 0;
      cp++;
   }
   else cp = s_name+strlen(s_name);

   strcpy(d_name,s_name);
   strupr(d_name);
   strcpy(des_name,d_name);
   strcat(d_name,".CNT");
   strcat(des_name,".DES");

   typ = 0;
   if(*cp)
   {
      strupr(cp);
      if(!strcmp(cp,"BIN")) typ = QW;
      if(!strcmp(cp,"KLG")) typ = KLOG;
   };

   if(!typ)
   {
      printf("\nDer Typ der Quelldatei kann nicht festgestellt werden!\n");
      return;
   };

   strcpy(s_name,argv[1]);

   f = fopen(s_name,typ==KLOG?"rt":"rb");
   if(f == NULL)
   {
      printf("\n Datei %s nicht gefunden!\n",s_name);
      return;
   };

   if(typ==QW)
   {
      fseek(f,22,SEEK_SET);
      fread(buf,1,3,f);
      if((buf[0]=='V') && (buf[2]=='.')) typ = UKWTEST;
      fseek(f,0,SEEK_SET);
   };

   if(typ==UKWTEST) 
   {
      if(argc<3)
      {
         memset(&bands,0,sizeof(bands));
         printf("\nBei dem angegebenen File handelt es sich um ein UKWTEST-File!\n\n");
         printf("Hierfr mu ein Bandcode angegeben werden. Nur QSOs auf diesem Band werden aus\n");
         printf("dem File extrahiert. Wenn man das entstandene CNT-File nach dem Extrahieren um-\n");
         printf("benennt, kann man nach und nach alle Bnder aus dem UKWTEST-File herausholen.\n");
         printf("Das DES-File mu dann manuell korrigiert werden!\n\n");
         printf("Mgliche Codes:\n");
   
         fseek(f,1144,SEEK_SET);
         read = fread(buf,1,60,f);
         while(read==60)
         {
            sel_band = buf[39];
            if(!bands[sel_band])
            {
               printf("%03d  ",sel_band);
               bands[sel_band] = 1;
            };
            read = fread(buf,1,60,f);
         };
         printf("\n\nAufruf mit: BABEL <Dateiname> <Code>\n\n");
         fclose(f);
         return;
      };

      sel_band = atoi(argv[2]);
   };
      
   outf = fopen(d_name,"wb");

   switch(typ)
   {
      case QW:
         fread(buf,1,1240,f);

         strcpy(h.magic,"CNT2\x1A");

         slen = buf[14];
         memmove(h.owncall,&buf[15],slen);
         h.owncall[slen] = 0;

         slen = buf[29];
         memmove(h.ownloc,&buf[30],slen);
         h.ownloc[slen] = 0;

         strcpy(h.desc,"Contest imported from QW");
         break;

      case UKWTEST:
         fread(buf,1,1144,f);

         strcpy(h.magic,"CNT2\x1A");

         memmove(h.owncall,&buf[0],13);
         memmove(h.ownloc,&buf[13],7);
         strcpy(h.desc,"Contest imported from UKWTEST");
         break;

      case KLOG:
         fgets(buf,256,f);

         strcpy(h.magic,"CNT2\x1A");

         cp = buf+8;
         while(isdigit(*cp) || isalpha(*cp) || (*cp=='/')) cp++;
         *cp = 0;
         strcpy(h.owncall,buf+8);

         strncpy(h.ownloc,buf+36,6);
         h.ownloc[6] = 0;

         strcpy(h.desc,"Contest imported from K-LOG");
         break;
   };

   loc2coord(h.ownloc,&own_north,&own_east);
  
   h.squares = 0;
   h.qso_cnt = 0;
   h.invalid = 0;
   h.points = 0;
   h.ODX = 0;
   h.countries = 0;
   h.unknown_pref = 0;

   first = NULL;
   last_nr = 0;

   fwrite(&h,1,sizeof(struct header),outf);

   while(!feof(f))
   {
      switch(typ)
      {
         case QW:
            read = fread(buf,1,62,f);
            ok = 0;

            if(read == 62)
            {
               ok = 1;
               q.nr = (buf[1]<<8)+buf[0];
 
               q.hour = ((buf[3]-'0')*10)+(buf[4]-'0');
               q.min = ((buf[5]-'0')*10)+(buf[6]-'0');
 
               slen = buf[14];
               if(buf[15] == '?')
               {
                  valid = 0;
                  slen--;
                  memmove(q.call,&buf[16],slen);
               }
               else
               {
                  valid = 1;
                  memmove(q.call,&buf[15],slen);
               };
               q.call[slen] = 0;
 
               slen = buf[29];
               memmove(q.locator,&buf[30],slen);
               q.locator[slen] = 0;
 
               rst_tx = (buf[38]<<8)+buf[37];
               sprintf(q.rst_TX,"%3ld",rst_tx);
 
               rst_rx = ((unsigned long)buf[44]<<24)+((unsigned long)buf[43]<<16)+((unsigned long)buf[42]<<8)+buf[41];
               if(rst_rx<60000) sprintf(q.rst_RX,"%3ld %03ld",rst_rx/1000L,rst_rx%1000L);
               else
               {
                  rst = rst_rx/1000L;
                  if((rst%10) == 1) sprintf(q.rst_RX,"%7ld",rst_rx);
                  else sprintf(q.rst_RX,"%3ld %03ld",(unsigned long)rst_rx/1000,(unsigned)rst_rx%1000);
               };
            };
            break;

         case UKWTEST:
            read = fread(buf,1,60,f);
            ok = 0;
            valid = 0;
            if(read==60)
            {
               ok = 1;
               memmove(&rst,&buf[0],4);
               t = gmtime(&rst);
               q.hour = ++t->tm_hour;
               q.min = t->tm_min;
               memmove(q.call,&buf[4],13);
               cp = q.call+12;
               while((*cp==' ')||(*cp=='\0')) cp--;
               cp++;
               *cp = 0;
               memmove(q.rst_TX,&buf[17],4);
               cp = q.rst_TX+3;
               while((*cp==' ')||(*cp=='\0')) cp--;
               cp++;
               *cp = 0;
               memmove(&q.nr,&buf[21],2);
               memmove(q.rst_RX,&buf[23],7);
               cp = q.rst_RX+6;
               while((*cp==' ')||(*cp=='\0')) cp--;
               cp++;
               *cp = 0;
               memmove(q.locator,&buf[30],7);
               band = buf[39];

               valid = 1;
               ok = (sel_band==band) && (last_nr != q.nr);
               if(ok) last_nr = q.nr;
            };   
            break;

         case KLOG:
            cp = fgets(buf,256,f);

            ok = 0;
            if(cp)
            {
               ok = strlen(buf)>30;
               q.hour = *(buf+3)-'0';
               q.min = *(buf+4)-'0';

               strncpy(q.call,buf+8,14);
               q.call[14] = 0;
               cp = q.call+13;
               while(*cp == ' ') cp--;
               *(cp+1) = 0;

               strncpy(s,buf+22,7);
               s[7] = 0;
            
               rst_tx = 0L;
               rst_tx = (unsigned long)atol(s);
               if(rst_tx>60000)
               {
                  rst = rst_tx/1000L;
                  if(rst%10==1)
                  {
                     q.nr = (unsigned)(rst_tx%10000L);
                     rst = (unsigned)(rst_tx/10000L);
                     sprintf(q.rst_TX,"%3ld",rst);
                  }
                  else
                  {
                     q.nr = (unsigned)(rst_tx%1000L);
                     rst = (unsigned)(rst_tx/1000L);
                     sprintf(q.rst_TX,"%3ld",rst);
                  };
               }
               else
               {
                  q.nr = (unsigned)(rst_tx%1000L);
                  rst = (unsigned)(rst_tx/1000L);
                  sprintf(q.rst_TX,"%3ld",rst);
               };

               strncpy(s,buf+29,7);
               s[7] = 0;

               rst_rx = 0L;
               rst_rx = (unsigned long)atol(s);
               if(rst_rx>60000)
               {
                  rst = rst_rx/1000L;
                  if(rst%10==1)
                  {
                     rst = (unsigned)(rst_rx/10000L);
                     sprintf(q.rst_RX,"%3ld%4ld",rst,rst_rx%10000L);
                  }
                  else
                  {
                     rst = (unsigned)(rst_rx/1000L);
                     sprintf(q.rst_RX,"%3ld %03ld",rst,rst_rx%1000L);
                  };
               }
               else
               {
                  rst = (unsigned)(rst_rx/1000L);
                  sprintf(q.rst_RX,"%3ld %03ld",rst,rst_rx%1000L);
               };

               strncpy(q.locator,buf+36,6);
               q.locator[6] = 0;

               valid = (*(buf+47)!='D');
            };
            break;
      };

      if(ok)
      {
         splitcall(q.call,pref,q.barecall,add);

         q.points = 0;
         q.direction = 361;
 
         if(valid)
         {
            for(hlp = first;hlp&&(strcmp(hlp->call,q.barecall));hlp = hlp->next) ;
 
            if(hlp)
            {
               valid = 0;
            }
            else
            {
               hlp = malloc(sizeof(struct callsign));
               hlp->next = first;
               strcpy(hlp->call,q.barecall);
               first = hlp;
            };
         };

         if(valid)
         {
            h.qso_cnt++;

            for(i=0;i<h.squares && (strncmp(squares[i],q.locator,4));i++);
 
            if(i==h.squares)
            {
               memmove(squares[i],q.locator,4);
               squares[i][5] = 0;
               h.squares++;
            };
 
            if(!strcmp(h.ownloc,q.locator))
            {
               q.points = 3;
               q.direction = 361;
            }
            else
            {
               loc2coord(q.locator,&north,&east);
               dist_and_dir(own_north,own_east,north,east,&dist,&dir);
               q.points = (unsigned)(dist+0.5);
               q.direction = (unsigned)(dir+0.5);
            };
 
            h.points+=q.points;

            if(q.points > h.ODX)
            {
               h.ODX = q.points;
               strcpy(h.ODX_call,q.call);
            };
         }
         else h.invalid++;
 
         fwrite((void *)&q.hour,1,2,outf);
         fwrite((void *)&q.min,1,2,outf);
         fwrite((void *)&q.nr,1,2,outf);
         fwrite((void *)q.barecall,1,13,outf);
         fwrite((void *)q.call,1,14,outf);
         fwrite((void *)q.locator,1,7,outf);
         fwrite((void *)q.rst_RX,1,8,outf);
         fwrite((void *)q.rst_TX,1,4,outf);
         fwrite((void *)&q.points,1,2,outf);
         fwrite((void *)&q.direction,1,2,outf);
      };
   };
   h.average = 0;
   if(h.qso_cnt) h.average = (unsigned)(h.points/h.qso_cnt);

   fseek(outf,SEEK_SET,0);
   fwrite(&h,1,sizeof(struct header),outf);

   fclose(f);
   fclose(outf);
   printf("\nDatei %s geschrieben.\n",d_name);

   while(first)
   {
      hlp = first;
      first = first->next;
      free(hlp);
   };

   outf = fopen(des_name,"wt");
   fprintf(outf,"TITLE Import\n");
   fprintf(outf,"DATE 010195\n");
   fprintf(outf,"Band = %s\n",d_name);
   fclose(outf);
   printf("\nBeschreibungsdatei %s geschrieben.\n",des_name);
   printf("Bitte mit einem ASCII-Editor editieren, um richtige Werte fr Titel und Datum\nanzugeben.\n\n");
   printf("Ich habe die von diesem Programm konvertierten Formate durch \"Aufschlsseln\"\n");
   printf("von Beispieldateien eingebaut. Es ist sehr wahrscheinlich, da es Flle gibt,\n");
   printf("in denen dieses Programm versagt und \"Mll\" konvertiert.\n\n");
   printf("Ebenso kann es sein, da sich die Dateiformate inzwischen gendert haben, oder\n");
   printf("einmal gendert werden. Auch dann versagt dieses Programm.\n\n");
   printf("HINWEIS: In dem erzeugten CNT-File ist die Anzahl der gearbeiteten Lnder noch\n");
   printf("nicht eingetragen. Dies wird automatisch korrigiert, sobald man in AUSWERT.EXE\n");
   printf("unter dem Oberpunkt \"Statistik...\" den Eintrag \"Lnderstatistik\" auswhlt.\n");
   printf("Beim nchsten Start von AUSWERT.EXE oder FIVENINE.EXE stimmt der Wert dann.\n\n");

   return;
};

