// data.C
// Written by Stefan Pfeiffer, DL1ELY, 960317
// Last modification: 960828
// (C) 1996 by Stefan Pfeiffer, DL1ELY@DB0IZ

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wildmat.h"
#include "locator.h"

typedef unsigned char byte;
typedef unsigned long ulong;
typedef unsigned int uint;

struct db_header
{
  byte version;  // 0x02 = DBase II, 0x03 = DBase III (0x83 = with memo)
  byte last_access[3];
  ulong rec_count;
  uint header_len;
  uint rec_len;
  byte reserved[20];
};

struct db_rec_def
{
  byte name[11];
  byte type;
  ulong address;
  byte len;
  byte decimals;
  byte reserved[14];
};

typedef struct call_node
{
   char call[13];
   unsigned long pos;
   struct call_node *left;
   struct call_node *right;
};

typedef struct call_index
{
   unsigned char call[12];
   long pos;
};

typedef struct loc_node
{
   char loc[7];
   unsigned long pos;
   struct loc_node *left;
   struct loc_node *right;
};

typedef struct loc_index
{
   char loc[6];
   unsigned long pos;
};

typedef struct multi_node
{
   char multi[7];
   unsigned long pos;
   struct multi_node *left;
   struct multi_node *right;
};

typedef struct multi_index
{
   char multi[6];
   unsigned long pos;
};

typedef struct callinfo
{
   unsigned char del;
   unsigned char call[13];
   unsigned char qrv_on[16];
   unsigned char locator[35];
   unsigned loc_date;
   unsigned char multi[11];
   unsigned multi_date;
   unsigned char info[128];
   unsigned long info_idx;
};

static int Star(s,p,argv)
register char *s;
register char *p;
register char **argv;
{
        char *cp = s;
        while (wildmat(cp, p, argv) == FALSE)
                if(*++cp == '\0')
                        return -1;
        return cp - s;
}

int wildmat(s,p,argv)
register char *s;
register char *p;
register char **argv;
{
        register int last;
        register int matched;
        register int reverse;
        register int cnt;

        for(; *p; s++,p++){
                switch(*p){
                case '\\':
                        /* Literal match with following character; fall through. */
                        p++;
                default:
                        if(*s != *p)
                                return FALSE;
                        continue;
                case '?':
                        /* Match anything. */
                        if(*s == '\0')
                                return FALSE;
                        continue;
                case '*':
                        /* Trailing star matches everything. */
                        if(argv == (char **)0)
                                return *++p ? 1 + Star(s, p, (char **)0) : TRUE;
                        if(*++p == '\0'){
                                cnt = strlen(s);
                        } else {
                                if((cnt = Star(s, p, argv+1)) == -1)
                                        return FALSE;
                        }
                        *argv = malloc(cnt+1);
                        strncpy(*argv,s,cnt);
                        *(*argv + cnt) = '\0';
                        return TRUE;
                case '[':
                        /* [^....] means inverse character class. */
                        reverse = (p[1] == '^') ? TRUE : FALSE;
                        if(reverse)
                                p++;
                        for(last = 0400, matched = FALSE; *++p && *p != ']'; last = *p){
                                /* This next line requires a good C compiler. */
                                if(*p == '-' ? *s <= *++p && *s >= last : *s == *p)
                                        matched = TRUE;
                        }
                        if(matched == reverse)
                                return FALSE;
                        continue;
                }
        }
        /* For "tar" use, matches that end at a slash also work. --hoptoad!gnu */
        return *s == '\0' || *s == '/';
}


char match(char *s, char *p)
{
   unsigned j;
   char *argv[50];
   char result;

   memset(&argv,0,sizeof(argv));
   result = wildmat(s,p,(char **)argv);
   for(j = 0;argv[j] != (char *)NULL;++j) free(argv[j]);

   return(result);
};


// Liest einen Calleintrag aus dem (bereits geffneten)
// File f, und liefert ihn in call_rec zurck.
void read_entry(FILE *f, FILE *f2, struct callinfo *call_rec)
{
   // Puffer fr einen DBF-Eintrag
   char buf[83];
   // Puffer fr den Infoeintrag zur Station
   char sbuf[80];


   fread(buf,1,83,f);   // DBF-Record einlesen.

   // Daten extrahieren
   call_rec->del = buf[0];
   memmove(call_rec->call,&buf[1],12);
   call_rec->call[12] = 0;

   memmove(sbuf,&buf[13],7);
   sbuf[7] = 0;
   call_rec->info_idx = atol(sbuf);

   memmove(call_rec->qrv_on,&buf[20],15);
   call_rec->qrv_on[15] = 0;

   memmove(call_rec->locator,&buf[35],34);
   call_rec->locator[34] = 0;

   memmove(sbuf,&buf[69],4);
   sbuf[4] = 0;
   call_rec->loc_date = atoi(sbuf);

   memmove(call_rec->multi,&buf[73],6);
   call_rec->multi[6] = 0;

   memmove(sbuf,&buf[79],4);
   sbuf[4] = 0;
   call_rec->multi_date = atoi(sbuf);

   // Infostring bearbeiten
   call_rec->info[0] = 0;
   if(call_rec->info_idx)
   {
      fseek(f2,call_rec->info_idx,SEEK_SET);
      fread(sbuf,1,80,f2);
      strcpy(call_rec->info,sbuf);
   };
};

// Schreibt den Stationseintrag call_rec in das (bereits
// geffnete) File f, und den evtl. Informationseintrag in
// das (geffnete) File f2.
void write_entry(FILE *f, FILE *f2, struct callinfo call_rec)
{
   // Puffer fr den DBF-Record
   unsigned char buf[84];
   // Offset des Info-Strings
   unsigned long pos = 0;
   unsigned char sbuf[20];


   // Evtl. vorhandenen Infoeintrag schreiben.
   if(call_rec.info[0])
   {
      fseek(f2,0,SEEK_END);
      pos = ftell(f2);
      fwrite(call_rec.info,1,strlen(call_rec.info)+1,f2);
   };

   // Puffer mit Daten fllen
   memset(buf,0,83);
   buf[0] = call_rec.del;

   // Call
   strcpy(&buf[1],call_rec.call);

   // Info-Index
   sprintf(sbuf,"%07d",pos);
   strcpy(&buf[13],sbuf);

   // QRV-String
   strcpy(&buf[20],call_rec.qrv_on);
   // Locators
   strcpy(&buf[35],call_rec.locator);

   // Datum des Loc-Eintrags
   sprintf(sbuf,"%04d",call_rec.loc_date);
   strcpy(&buf[69],sbuf);

   // Multi-Info
   strcpy(&buf[73],call_rec.multi);
 
   // Datum des Multi-Eintrags
   sprintf(sbuf,"%04d",call_rec.multi_date);
   strcpy(&buf[79],sbuf);

   // Puffer schreiben
   fwrite(buf,1,83,f);

   return;
};



unsigned long call_bin_search(FILE *f, char *call)
{
   unsigned cnt;
   unsigned l, r;
   unsigned look_at;
   struct call_index ind;
   int res;

   fseek(f,0,SEEK_END);
   cnt = (ftell(f)-8)/sizeof(struct call_index);

   l = 0;
   r = cnt;

   while(r-l>1)
   {
      look_at = (r+l)>>1;
      fseek(f,((unsigned long)look_at*sizeof(struct call_index))+8,SEEK_SET);
      fread(&ind,1,sizeof(struct call_index),f);

      res = strcmp(ind.call,call);
      
      if(!res)
      {
         r = look_at;
         l = look_at;
      };

      if(res>0) r = look_at;

      if(res<0) l = look_at;
   };

   if(r==l) return(ind.pos);
   else return(0L);
};


struct call_node *insert_call(struct call_node *root,struct call_node *ins)
{
   ins->left = NULL;
   ins->right = NULL;

   if(!root) return(ins);

   if(strcmp(root->call,ins->call)>0)
   {
      root->left = insert_call(root->left,ins);
      return(root);
   }
   else
   {
      root->right = insert_call(root->right,ins);
      return(root);
   };
};


void inorder_call(FILE *f,struct call_node *root)
{
   if(root)
   {
      if(root->left) inorder_call(f,root->left);
      printf("%14s\r",root->call);
      fwrite(root->call,1,12,f);
      fwrite(&root->pos,1,4,f);
      if(root->right) inorder_call(f,root->right);
   };
};

void delete_call(struct call_node *root)
{
   if(root)
   {
      delete_call(root->left);
      delete_call(root->right);
      free(root);
   };
};


void sort_calls(FILE *f)
{
   struct call_index call[32];
   struct call_node *root = NULL;
   struct call_node *new_node;
   unsigned read, i;
   char s[9];


   fseek(f,0,SEEK_SET);
   read = 32;
   while(read == 32)   
   {
      read = fread(call,sizeof(struct call_index),32,f);
      for(i=0;i<read;i++)
      {
         new_node = malloc(sizeof(struct call_node));
         if(!new_node) abort();

         memcpy(new_node->call,call[i].call,12);
         new_node->call[12] = 0;
         new_node->pos = call[i].pos;

         new_node->left = NULL;
         new_node->right = NULL;

         root = insert_call(root,new_node);
      };
   };

   fseek(f,0,SEEK_SET);
   strcpy(s,"INDEX   ");
   fwrite(s,1,8,f);
   inorder_call(f,root);

   delete_call(root);
   root = NULL;
};
    

struct loc_node *insert_loc(struct loc_node *root,struct loc_node *ins)
{
   int result;

   if(!root) return(ins);

   result = strcmp(root->loc,ins->loc);  
   if(result>0)
   {
      root->left = insert_loc(root->left,ins);
      return(root);
   };
   if(result<0)
   {
      root->right = insert_loc(root->right,ins);
      return(root);
   };
   
   if(root->pos < ins->pos)
   {
      root->right = insert_loc(root->right,ins);
      return(root);
   };

   root->left = insert_loc(root->left,ins);
   return(root);
};


void inorder_loc(FILE *f,struct loc_node *root)
{
   if(root)
   {
      inorder_loc(f,root->left);
      fwrite(root->loc,1,6,f);
      fwrite(&root->pos,1,4,f);
      inorder_loc(f,root->right);
   };
};

void delete_loc(struct loc_node *root)
{
   if(root)
   {
      delete_loc(root->left);
      delete_loc(root->right);
      free(root);
   };
};


void sort_locs(FILE *f)
{
   struct loc_index loc[32];
   struct loc_node *root = NULL;
   struct loc_node *new_node;
   unsigned read, i;
   char s[9];


   fseek(f,0,SEEK_SET);
   read = 32;
   while(read == 32)   
   {
      read = fread(loc,sizeof(struct loc_index),32,f);
      for(i=0;i<read;i++)
      {
         new_node = malloc(sizeof(struct loc_node));
         memcpy(new_node->loc,loc[i].loc,6);
         new_node->loc[6] = 0;
         new_node->pos = loc[i].pos;

         new_node->left = NULL;
         new_node->right = NULL;

         root = insert_loc(root,new_node);
      };
   };

   fseek(f,0,SEEK_SET);
   strcpy(s,"INDEX   ");
   fwrite(s,1,8,f);
   inorder_loc(f,root);

   delete_loc(root);
   root = NULL;
};
           
         
struct multi_node *insert_multi(struct multi_node *root,struct multi_node *ins)
{
   int result;

   if(!root) return(ins);

   result = strcmp(root->multi,ins->multi);
   if(result>0)
   {
      root->left = insert_multi(root->left,ins);
      return(root);
   };

   if(result>0)
   {
      root->right = insert_multi(root->right,ins);
      return(root);
   };

   if(root->pos < ins->pos)
   {
      root->right = insert_multi(root->right,ins);
      return(root);
   };

   root->left = insert_multi(root->left,ins);
   return(root);
};


void inorder_multi(FILE *f,struct multi_node *root)
{
   if(root)
   {
      inorder_multi(f,root->left);
      fwrite(root->multi,1,6,f);
      fwrite(&root->pos,1,4,f);
      inorder_multi(f,root->right);
   };
};

void delete_multi(struct multi_node *root)
{
   if(root)
   {
      delete_multi(root->left);
      delete_multi(root->right);
      free(root);
   };
};


void sort_multis(FILE *f)
{
   struct multi_index multi[32];
   struct multi_node *root = NULL;
   struct multi_node *new_node;
   unsigned read, i;
   char s[9];


   fseek(f,0,SEEK_SET);
   read = 32;
   while(read == 32)   
   {
      read = fread(multi,sizeof(struct multi_index),32,f);
      for(i=0;i<read;i++)
      {
         new_node = malloc(sizeof(struct multi_node));
         memcpy(new_node->multi,multi[i].multi,6);
         new_node->multi[6] = 0;
         new_node->pos = multi[i].pos;

         new_node->left = NULL;
         new_node->right = NULL;

         root = insert_multi(root,new_node);
      };
   };

   fseek(f,0,SEEK_SET);
   strcpy(s,"INDEX   ");
   fwrite(s,1,8,f);
   inorder_multi(f,root);

   delete_multi(root);
   root = NULL;
};
  
void do_reorg(char *stn_path)
{
   char stn_dbf_name[128];
   char stn_nfo_name[128];
   char call_dex_name[128];
   char loc_dex_name[128];
   char multi_dex_name[128];
   char temp_dbf_name[128];
   char temp_nfo_name[128];

   FILE *stn_dbf;
   FILE *stn_nfo;
   FILE *call_dex;
   FILE *loc_dex;
   FILE *multi_dex;
   FILE *temp_dbf;
   FILE *temp_nfo;

   struct db_header dbf_head;
   struct db_rec_def field_rec;

   unsigned written, killed, i, ok, max_call;
   char s[20];

   struct callinfo call_rec;

   unsigned long pos;


   strcpy(stn_dbf_name,stn_path);
   strcat(stn_dbf_name,"STATIONS.DBF");

   strcpy(stn_nfo_name,stn_path);
   strcat(stn_nfo_name,"STATIONS.NFO");

   strcpy(temp_dbf_name,stn_path);
   strcat(temp_dbf_name,"TEMP$$$$.DBF");

   strcpy(temp_nfo_name,stn_path);
   strcat(temp_nfo_name,"TEMP$$$$.NFO");

   strcpy(call_dex_name,stn_path);
   strcat(call_dex_name,"STATIONS.DEX");

   strcpy(loc_dex_name,stn_path);
   strcat(loc_dex_name,"LOCATORS.DEX");

   strcpy(multi_dex_name,stn_path);
   strcat(multi_dex_name,"MULTIS.DEX");

   ok = 1;
   if((stn_dbf = fopen(stn_dbf_name,"rb")) == NULL) ok = 0;
   if((stn_nfo = fopen(stn_nfo_name,"rb")) == NULL) ok = 0;
   if((temp_dbf = fopen(temp_dbf_name,"wb")) == NULL) ok = 0;
   if((temp_nfo = fopen(temp_nfo_name,"wb")) == NULL) ok = 0;
   if((call_dex = fopen(call_dex_name,"wb+")) == NULL) ok = 0;
   if((loc_dex = fopen(loc_dex_name,"wb+")) == NULL) ok = 0;
   if((multi_dex = fopen(multi_dex_name,"wb+")) == NULL) ok = 0;

   if(!ok)
   {
      printf("Cannot open a file!\n\n");
      return;
   };

   // Einlesen des DBF-Headers
   fread(&dbf_head,1,sizeof(struct db_header),stn_dbf);

   max_call = dbf_head.rec_count;

   // bertragen des DBF-Headers in die neue Datei
   dbf_head.rec_count = 0;
   fwrite(&dbf_head,1,sizeof(struct db_header),temp_dbf);

   // Kopieren der Datenfelder der DBF-Datei
   for(i=1;i<8;i++)
   {
      fread(&field_rec,1,sizeof(struct db_rec_def),stn_dbf);
      fwrite(&field_rec,1,sizeof(struct db_rec_def),temp_dbf);
   };

   // End-Markierung fr Felddefinition
   i = 0x0d;
   fwrite(&i,1,1,temp_dbf);

   // Header des Info-Files schreiben
   strcpy(s,"INFO    ");
   fwrite(s,1,8,temp_nfo);

   fseek(stn_dbf,dbf_head.header_len,SEEK_SET);

   written = 0;
   killed = 0;

   printf("     /%5d records\r",max_call);
   while(!feof(stn_dbf))
   {
      read_entry(stn_dbf,stn_nfo,&call_rec);

      if(call_rec.del == 0x20)
      {
         pos = ftell(temp_dbf);
         
         //write_call_ndx(call_dex,call_rec.call,pos);
         fwrite(call_rec.call,1,12,call_dex);
         fwrite(&pos,1,4,call_dex);

         if(call_rec.locator[0])
         {
            //write_loc_ndx(loc_dex,call_rec.locator,pos);
            fwrite(call_rec.locator,1,6,loc_dex);
            fwrite(&pos,1,4,loc_dex);
         };

         if(call_rec.multi[0])
         {
            //write_multi_ndx(multi_dex,call_rec.multi,pos);
            fwrite(call_rec.multi,1,6,multi_dex);
            fwrite(&pos,1,4,multi_dex);
         };

         write_entry(temp_dbf,temp_nfo,call_rec);
         written++;
         if(!(written%10)) printf("%5d\r",written);
      }
      else killed++;
   };

   printf("%5d\n",written);

   // Anzahl der Records in das DBF-File schreiben
   fseek(temp_dbf,4,SEEK_SET);
   pos = (long)written;
   fwrite(&pos,1,4,temp_dbf);

   // Dateien schlieen.
   fclose(temp_dbf);
   fclose(temp_nfo);
   fclose(stn_dbf);
   fclose(stn_nfo);

   //Temporrdateien umbennen
   unlink(stn_dbf_name);
   rename(temp_dbf_name,stn_dbf_name);

   unlink(stn_nfo_name);
   rename(temp_nfo_name,stn_nfo_name);

   printf("\nSorting calls...\n");
   sort_calls(call_dex);
   fclose(call_dex);

   printf("Sorting locators...\n");
   sort_locs(loc_dex);
   fclose(loc_dex);

   printf("Sorting multis...\n");
   sort_multis(multi_dex);
   fclose(multi_dex);
};

         
void main(int argc, char **argv)
{
   FILE *f;
   FILE *f2;
   FILE *f3;
   char scall[20];
   char stn_path[128];
   char s[128];
   char *cp;
   struct callinfo qso;
   unsigned long pos;
   struct call_index ndx[64];
   char i;
   unsigned int read;
   char search_end = 0;

   char own_loc[7];
   double own_north, own_east;
   double north, east;
   double dist, dir;

   char month[13][4] = {"\0","Jan","Feb","Mar","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

   printf("Database-Services v1.20 by Stefan Pfeiffer, DL1ELY@DB0AIS\n");
   if(argc<2)
   {
      printf("\n    DATA <callsign>, example: DATA dl1ely\n");
      printf("or: DATA <callmask>, example: DATA DL1E*\n\n");
      printf("    DATA REORG does a reorganisation of the database.\n\n"); 
      printf("STATIONS.DBF and STATIONS.DEX are expected to reside in the same directory as\n");
      printf("DATA.EXE! If not, use the environment-variable STN_PATH to specify the path to\n");
      printf("the files, for example \"SET STN_PATH = C:\\FIVENINE\" in AUTOEXEC.BAT.\n\n");
      printf("Please put your locator in OWN_LOG, for example \"SET OWN_LOC = JO31IO\".\n\n");
      printf("This programme is freeware! See LICENCE.DOC for further details!\n\n");
      printf("Have fun and good DX! Vy 73 de Stefan, DL1ELY\n\n");

      exit(0);
   };

   cp = getenv("STN_PATH");
   if(cp)
   {
      strcpy(stn_path,cp);
      if(stn_path[strlen(stn_path)-1]!='\\') strcat(stn_path,"\\");
   }
   else
   {
      strcpy(stn_path,argv[0]);
      cp = stn_path+strlen(stn_path)-1;
      while(*cp!='\\') cp--;
      cp++;
      *cp = 0;
   };

   own_loc[0] = 0;
   cp = getenv("OWN_LOC");
   if(cp)
   {   
      while(*cp == ' ') cp++;
      memcpy(own_loc,cp,6);
      own_loc[6] = 0;
      strupr(own_loc);
      loc2coord(own_loc,&own_north,&own_east);
   };

   
   strcpy(scall,argv[1]);
   strupr(scall);

   if(!strcmp(scall,"REORG"))
   {
      do_reorg(stn_path);
      return(0);
   };

   strcpy(s,stn_path);
   strcat(s,"STATIONS.DEX");

   if((f=fopen(s,"rb"))==NULL)
   {
      printf("\nFile not found: %s\n",s);
      exit(0);
   };

   i = strchr(scall,'?') || strchr(scall,'*') || strchr(scall,'[');
   if(!i)
   {
      pos = call_bin_search(f,scall);
      search_end = 1;

      fclose(f);

      if(!pos)
      {
         printf("\nCall %s not in the database!\n",scall);
         exit(0);
      };
   };

   strcpy(s,stn_path);
   strcat(s,"STATIONS.DBF");
   if((f2 = fopen(s,"rb")) == NULL)
   {
      printf("\nFile not found: %s\n",s);
      exit(0);
   };

   strcpy(s,stn_path);
   strcat(s,"STATIONS.NFO");
   if((f3 = fopen(s,"rb")) == NULL)
   {
      fclose(f2);
      printf("\nFile not found: %s\n",s);
      exit(0);
   };

   i = 64;
   read = 0;
   fseek(f,8,SEEK_SET);
   do
   {
      if(!search_end)
      {
         i++;
         if(i>=read)   
         {
            read = fread(&ndx,sizeof(struct call_index),64,f);
            i = 0;
         };
         pos = 0;
         while((i<read) && !match(ndx[i].call,scall)) i++;
         if(i<read) pos = ndx[i].pos;
         if(read<64) search_end = 1;
      };

      if(pos)
      {
         fseek(f2,pos,SEEK_SET);
         read_entry(f2,f3,&qso);


         printf("\n%s",qso.call);
         if(qso.locator[0])
         {
            printf(" (%s %d): ",month[qso.loc_date%100],qso.loc_date/100);
            cp = qso.locator+6;
            if(*cp)
            {
               *cp = 0;
               cp++;
            };
            printf(qso.locator);
            if(own_loc[0])
            {
               loc2coord(qso.locator,&north,&east);
               dist_and_dir(own_north,own_east,north,east,&dist,&dir);
               printf(" (%dkm QRB) ",(unsigned)(dist+0.5));
            }
            else printf(" ");
            if(*cp) printf(cp);
         };

         if(qso.multi[0]) printf("  -  Multi (%s %d): %s",month[qso.multi_date%100],qso.multi_date/100,qso.multi);
         if(qso.info[0]) printf("  -  Info: %s\n",qso.info);
      };
   } while(!search_end);

   printf("\n");

   fclose(f);
   fclose(f2);
   fclose(f3);

   return;
};

