// database.c - Routines for databases in FiveNine
// Written by Stefan Pfeiffer, DL1ELY, 940407
// Last Modification: 951230
// (C) 1994 by Stefan Pfeiffer, DL1ELY

// Include definitions and other includefiles
#include "database.h"
#include "dbase.h"

// Root and last prefix of prefix-list
struct countryinfo *firstcountry = NULLcountry;
struct countryinfo *lastcountry = NULLcountry;

// Root and last QSO of QSO-list
struct qso *firstqso = NULLqso;
struct qso *lastqso = NULLqso;

// Root and last entry of city-list           // Not yet implemented!!
// struct cityinfo *firstcity = NULLcityinfo;
// struct cityinfo *lastcity = NULLcityinfo;

// Root and last entry of worked squares
struct squareinfo *firstsquare = NULLsquare;
struct squareinfo *lastsquare = NULLsquare;

// Header of read contest-file
struct header h;
unsigned char countries;
unsigned char pref_not_found;
unsigned invalid;

extern char *fn_path[128];

// Index der Rufzeichen
unsigned call_handle = 0;
unsigned char call_type;
unsigned call_cnt = 0;
FILE *call_file = NULL;

unsigned multi_handle = 0;
unsigned char multi_type;
unsigned multi_cnt = 0;
FILE *multi_file = NULL;

unsigned locator_handle = 0;
unsigned char locator_type;
unsigned locator_cnt = 0;
FILE *locator_file = NULL;


void my_gotoxy(unsigned char, unsigned char);


char qso_match(struct qso *q, struct qso_mask *qm)
{
   unsigned minutes;
   unsigned val;
   char i;
   char *av[20];
   char call_ok;

   if(q->nr < qm->min_nr) return(0);
   if(qm->max_nr && q->nr > qm->max_nr) return(0);

   if(q->hour > 13)
      minutes = (q->hour-14)*60 + q->min;
   else
      minutes = (q->hour+10)*60 + q->min;

   if(qm->min_hour > 13)
      val = (qm->min_hour-14)*60 + qm->min_minute;
   else
      val = (qm->min_hour+10)*60 + qm->min_minute;

   if(minutes < val) return(0);

   if(qm->max_hour > 13)
      val = (qm->max_hour-14)*60 + qm->max_minute;
   else
      val = (qm->max_hour+10)*60 + qm->max_minute;

   if(minutes > val) return(0);

   if(q->points < qm->min_qrb) return(0);

   if(qm->max_qrb && q->points > qm->max_qrb) return(0);

   if(q->direction < qm->min_deg) return(0);

   if(q->direction < 361 && q->direction > qm->max_deg) return(0);

   call_ok = 1;
   if(strcmp(qm->call,"*"))
   {
      memset(&av,0,sizeof(av));
      if(!wildmat(q->barecall,qm->call,av)) call_ok = 0;
    
      for(i = 0;av[i] != (char *)0;++i) sec_free(av[i]);

      if(!call_ok)
      {
         memset(&av,0,sizeof(av));
         call_ok = wildmat(q->call,qm->call,av);

         for(i = 0;av[i] != (char *)0;++i) sec_free(av[i]);
      };
   };

   if(!call_ok) return(0);

   if(strcmp(qm->loc,"*"))
   {
      memset(&av,0,sizeof(av));
      if(!wildmat(q->locator,qm->loc,av)) return(0);
    
      for(i = 0;av[i] != (char *)0;++i) sec_free(av[i]);
   };

   return(1);
};

void query_qso_mask(struct qso_mask *qm)
{
   void far *save;
   char res;
   char sel;
   char s[80];
   unsigned hlp;
   char ch;

   strcpy(qm->call,"*");
   strcpy(qm->loc,"*");
   qm->min_nr = 0;
   qm->max_nr = 0;
   qm->min_hour = 14;
   qm->min_minute = 0;
   qm->max_hour = 13;
   qm->max_minute = 59;
   qm->min_qrb = 0;
   qm->max_qrb = 0;
   qm->min_deg = 0;
   qm->max_deg = 360;

   save = dsave(0,0,79,lastline);

   dfill(0,0,79,0,' ',titlecol);
   dprint(32,0,titlecol,"  Select QSOs  ");
   dfill(0,1,79,lastline,' ',menucol);

   dprint(2,2,menucol,"Call-mask (with wildcards):   ");
   dfill(33,2,52,2,' ',menuinvcol);
   dprintf(33,2,menuinvcol,"%s",qm->call);

   dprint(2,4,menucol,"Locator-mask (with wildcards):   ");
   dfill(36,4,55,4,' ',menuinvcol);
   dprintf(36,4,menuinvcol,"%s",qm->loc);

   dprint(2,6,menucol,"contest-number:        above         and below or equal        (MAX=highest)");
   dfill(33,6,37,6,' ',menuinvcol);
   dprint(33,6,menuinvcol,"0");
   dfill(58,6,62,6,' ',menuinvcol);
   dprint(58,6,menuinvcol,"MAX");

   dprint(2,8,menucol,"Time:   later than     :    , earlier than    :");
   dfill(22,8,24,8,' ',menuinvcol);
   dprint(22,8,menuinvcol,"14");
   dfill(26,8,28,8,' ',menuinvcol);
   dprint(26,8,menuinvcol,"00");
   dfill(45,8,47,8,' ',menuinvcol);
   dprint(45,8,menuinvcol,"13");
   dfill(49,8,51,8,' ',menuinvcol);
   dprint(49,8,menuinvcol,"59");

   dprint(2,10,menucol,"Distance:   between        km and       km  (MAX=farest QSO)");

   dfill(23,10,27,10,' ',menuinvcol);
   dprint(23,10,menuinvcol,"0");
   dfill(36,10,40,10,' ',menuinvcol);
   dprint(36,10,menuinvcol,"MAX");

   dprint(2,12,menucol,"Angle:  between       deg. and      deg.");
   dfill(19,12,22,12,' ',menuinvcol);
   dprint(19,12,menuinvcol,"000");
   dfill(33,12,36,12,' ',menuinvcol);
   dprint(33,12,menuinvcol,"360");

   paint_but(35,15,"OK",passivebuttoncol,menucol);

   dfill(0,18,79,18,196,menucol);

   dprint(1,20,menucol,"Here you can select the QSOs you want to work with.");
   dprint(1,21,menucol,"All properties have to fit to a QSO SIMULTANOUSLY to be selected.");
   dprint(1,22,menucol,"(AND-conjunction of fields)");
   dprint(1,23,menucol,"The defaults will select ALL QSOs.");

   sel = 1;
   do
   {
      switch(sel)
      {
         case 1:
            res = input_string(33,2,19,0,DO_TAB|CALL_CONVERT|INSERT|UPCASE,menuinvcol,qm->call);
            break;
         case 2:
            res = input_string(36,4,19,0,DO_TAB|INSERT|UPCASE,menuinvcol,qm->loc);
            break;
         case 3:
            sprintf(s,"%u",qm->min_nr);
            res = input_string(33,6,4,0,DO_TAB|DIGITS_ONLY|INSERT,menuinvcol,s);
            if(res) qm->min_nr = atoi(s);
            break;
         case 4:
            strcpy(s,"MAX");
            if(qm->max_nr) sprintf(s,"%u",qm->max_nr);
            res = input_string(58,6,4,0,DO_TAB|INSERT|UPCASE,menuinvcol,s);
            if(res)
            {
               if(!strcmpi(s,"MAX")) qm->max_nr = 0;
               else
                  qm->max_nr = atoi(s);

               dfill(58,6,62,6,' ',menuinvcol);
               if(qm->max_nr) dprintf(58,6,menuinvcol,"%u",qm->max_nr);
               else dprint(58,6,menuinvcol,"MAX");
            };
            break;
         case 5:
            sprintf(s,"%02u",qm->min_hour);
            res = input_string(22,8,2,0,DO_TAB|INSERT|DIGITS_ONLY,menuinvcol,s);
            if(res)
            {
               hlp = atoi(s);
               if(hlp<24) qm->min_hour = hlp;
               dfill(22,8,24,8,' ',menuinvcol);
               dprintf(22,8,menuinvcol,"%02u",qm->min_hour);
            };
            break;
         case 6:
            sprintf(s,"%02u",qm->min_minute);
            res = input_string(26,8,2,0,DO_TAB|INSERT|DIGITS_ONLY,menuinvcol,s);
            if(res)
            {
               hlp = atoi(s);
               if(hlp<60) qm->min_minute = hlp;
               dfill(26,8,28,8,' ',menuinvcol);
               dprintf(26,8,menuinvcol,"%02u",qm->min_minute);
            };
            break;
         case 7:
            sprintf(s,"%02u",qm->max_hour);
            res = input_string(45,8,2,0,DO_TAB|INSERT|DIGITS_ONLY,menuinvcol,s);
            if(res)
            {
               hlp = atoi(s);
               if(hlp<24) qm->max_hour = hlp;
               dfill(45,8,47,8,' ',menuinvcol);
               dprintf(45,8,menuinvcol,"%02u",qm->max_hour);
            };
            break;
         case 8:
            sprintf(s,"%02u",qm->max_minute);
            res = input_string(49,8,2,0,DO_TAB|INSERT|DIGITS_ONLY,menuinvcol,s);
            if(res)
            {
               hlp = atoi(s);
               if(hlp<60) qm->max_minute = hlp;
               dfill(49,8,51,8,' ',menuinvcol);
               dprintf(49,8,menuinvcol,"%02u",qm->max_minute);
            };
            break;
         case 9:
            sprintf(s,"%u",qm->min_qrb);
            res = input_string(23,10,4,0,DO_TAB|INSERT|DIGITS_ONLY,menuinvcol,s);
            if(res) qm->min_qrb = atoi(s);
            break;
         case 10:
            strcpy(s,"MAX");
            if(qm->max_qrb) sprintf(s,"%u",qm->max_qrb);
            res = input_string(36,10,4,0,DO_TAB|INSERT|UPCASE,menuinvcol,s);
            if(res)
            {
               if(!strcmpi(s,"MAX")) qm->max_qrb = 0;
               else qm->max_qrb = atoi(s);
 
               dfill(36,10,40,10,' ',menuinvcol);
               if(qm->max_qrb) dprintf(36,10,menuinvcol,"%u",qm->max_qrb);
               else dprint(36,10,menuinvcol,"MAX");
            };
            break;
         case 11:
            sprintf(s,"%03u",qm->min_deg);
            res = input_string(19,12,3,0,DO_TAB|INSERT|DIGITS_ONLY,menuinvcol,s);
            if(res) qm->min_deg = atoi(s);
            if(qm->min_deg > 359)
            {
               qm->min_deg = 360;
               dprint(19,12,menuinvcol,"360");
            };
            break;
         case 12:
            sprintf(s,"%03u",qm->max_deg);
            res = input_string(33,12,3,0,DO_TAB|INSERT|DIGITS_ONLY,menuinvcol,s);
            if(res) qm->max_deg = atoi(s);
            if(qm->max_deg > 359)
            {
               qm->max_deg = 360;
               dprint(33,12,menuinvcol,"360");
            };
            break;
         case 13:
            paint_but(35,15,"OK",activebuttoncol,menucol);
            my_gotoxy(80,lastline+1);
            ch = getch();
            if((ch == 27) || (ch == 13)) res = 0;
            if(ch == 9)
            {
               res = TAB;
               paint_but(35,15,"OK",passivebuttoncol,menucol);
            };
            if(!ch)
            {
               ch = getch();
               if(ch == 15)
               {
                  res = BACKTAB;
                  paint_but(35,15,"OK",passivebuttoncol,menucol);
               };
            };
            break;
      };

      if((res == 1)||(res == TAB))
      {
         sel++;
         if(sel>13) sel = 1;
         res = 2;
      };
      if(res == BACKTAB)
      {
         sel--;
         if(!sel) sel = 13;
         res = 2;
      };
   } while(res == 2);
        
   drestore(save);
   sec_free(save);

   my_gotoxy(80,lastline+1);
};


// Shows all entries containing a specific locator from the database
void display_base_locs()
{
   void far *save;
   uchar searchloc[7];

   FILE *f;

   return;
};


// Splits a given call in its parts
void splitcall(pchar rawcall,pchar prefix,pchar call,pchar add)
{
   uchar hlp[81];
   pchar cp1;
   pchar cp2;

   *prefix = 0;
   *call = 0;
   *add = 0;

   cp1 = rawcall;
   cp2 = hlp;
   while(*cp1 && *cp1 != '/')
   {
      *cp2 = *cp1;
      cp1++;
      cp2++;
   }
   *cp2 = 0;

   if(!hlp[0]) return;

   if(strlen(hlp) > 3)
   {
      cp1 = rawcall;
   }
   else
   {
      strcpy(prefix,hlp);
      if(*cp1) cp1++;
   }

   cp2 = hlp;
   while(*cp1 && *cp1 != '/')
   {
      *cp2 = *cp1;
      cp1++;
      cp2++;
   }
   *cp2 = 0;

   if((strlen(hlp) < 4) || (strlen(hlp) > 12)) return;
   strcpy(call,hlp);

   if(!*cp1)
   {
      *add = 0;
   }
   else
   {
      cp1++;
      if(strlen(cp1) > 2)
      {
         *call = 0;
         return;
      }
      strcpy(add,cp1);
   }
};

// Generates a "normal" call
void make_norm_call(pchar call, pchar normcall)
{
   char pref[4];
   char barecall[13];
   char add[3];

   splitcall(call,pref,barecall,add);
   normcall[0] = 0;
   if(!barecall[0]) return;

   if(pref[0])
   {
      strcpy(normcall,pref);
      strcat(normcall,"/");
   };
   strcat(normcall,barecall);
   if(isdigit(add[0]) && !add[1])
   {
      strcat(normcall,"/");
      strcat(normcall,add);
   };
};

// Loads a QSO from a CNT-File
uchar load_next_qso(FILE *f, struct qso far *q)
{
   unsigned read;
  
   memset((void *)q,0,QSO_SIZE);

   read = 0;
   read+=fread((void *)&q->hour,1,2,f);
   read+=fread((void *)&q->min,1,2,f);
   read+=fread((void *)&q->nr,1,2,f);
   read+=fread((void *)q->barecall,1,13,f);
   read+=fread((void *)q->call,1,14,f);
   read+=fread((void *)q->locator,1,7,f);
   read+=fread((void *)q->rst_RX,1,8,f);
   read+=fread((void *)q->rst_TX,1,4,f);
   read+=fread((void *)&q->points,1,2,f);
   read+=fread((void *)&q->direction,1,2,f);
  
   return(read==56);
};  

// Inserts a QSO in the statistics
void make_stat(struct qso q)
{ 
   char pref[4];
   char call[7];
   char add[3];
   struct countryinfo *prefix;
   struct squareinfo *actual_square;

   splitcall(q.call,pref,call,add);
   if(*pref) prefix = find_prefix(pref);
   else prefix = find_prefix(call);
   if(prefix)
   {
      if(!prefix->worked) countries++;
      prefix->worked++;
      prefix->points += q.points;
   }
   else pref_not_found++;

   if(!q.points) invalid++;

   actual_square = find_square(q.locator);

   if(!actual_square) actual_square = add_square(q.locator);
   actual_square->worked++;
   actual_square->points += q.points;
};


// Reads the given file
void open_contest_file(pchar filename, pchar bandname)
{
   struct qso load_qso;
   FILE *cntfile;
   void *save;
   unsigned cnt, schwelle;

   error_log("open_contest_file()","Opening window");
   save = make_win(10,5,70,10,menucol);
   dprintf(35-(strlen(bandname)>>1),6,menucol,"read QSOs: %s",bandname);

   error_log("open_contest_file()","Reading contest header...");
   if(read_contest_info(filename))
   {
      error_log("open_contest_file()","Invalid header");
      drestore(save);
      sec_free(save);
      return;
   };

   cntfile = fopen(filename,"rb");
   fseek(cntfile,HEADER_SIZE,SEEK_SET);

   countries = 0;
   pref_not_found = 0;
   invalid = 0;
   cnt = 0;
   init_bar(9,menucol,h.qso_cnt);

   error_log("open_contest_file()","Start reading %s",filename);
   while(load_next_qso(cntfile,&load_qso))
   {
      paint_bar(cnt);
      cnt++;

      if(load_qso.nr > 0)
      {
         add_qso(load_qso);
         make_stat(load_qso);
      }
   }

   error_log("open_contest_file()","Freeing window");
   drestore(save);
   sec_free(save);

   fclose(cntfile);
   if((invalid!=h.invalid) || (pref_not_found!=h.unknown_pref) || (countries!=h.countries))
   {
      error_log("open_contest_file()","Adjusting header");
      h.unknown_pref = pref_not_found;
      h.countries = countries;
      h.invalid = invalid;
      cntfile = fopen(filename,"r+b");
      fwrite((void *)&h,1,HEADER_SIZE,cntfile);
      fclose(cntfile);
   };

   return;
};

// Reads the header of a contest-file
uchar read_contest_info(pchar filename)
{
   FILE *cntfile;

   error_log("read_contest_info()","Called");
   if((cntfile = fopen(filename,"rb")) == NULLfile)
   {
      error_log("read_contest_info()","File %s not found",filename);
      return(FILE_NOT_FOUND);
   };

   fread((void *)&h,HEADER_SIZE,1,cntfile);
   fclose(cntfile);

   if(strncmp(h.magic,"CNT2\x1A",5))
   {
      memset((void *)&h,0,HEADER_SIZE);
      error_log("read_contest_info()","Invalid ID");
      return(INVALID_ID);
   };
   return(0);
};

// Writes all QSOs of the QSO-list in the stated file with the given header
uchar write_contest_file(pchar filename, struct header *head, pchar name)
{
   struct qso *q;
   FILE *tmpfile;
   int written;
   void *save;
   unsigned cnt, schwelle;

   error_log("write_contest_file()","Called");
   if((tmpfile = fopen(TMP_NAME,"wb"))==NULLfile) return(OPEN_FILE_ERR);

   error_log("write_contest_file()","Writing header");
   written = fwrite((void *)head,HEADER_SIZE,1,tmpfile);
   if(!written)
   {
      fclose(tmpfile);
      unlink(TMP_NAME);
      return(DISK_FULL_ERR);
   };

   error_log("write_contest_file()","Opening window");
   save = make_win(10,5,70,10,menuinvcol);
  
   dprintf(33-(strlen(name)>>1),6,menuinvcol," writing QSOs: %s",name);
 
   error_log("write_contest_file()","Start writing");
   q = firstqso;
   cnt = 0;

   init_bar(9,menucol,h.qso_cnt);

   while(q)
   {
      fwrite((void *)&q->hour,1,2,tmpfile);
      fwrite((void *)&q->min,1,2,tmpfile);
      fwrite((void *)&q->nr,1,2,tmpfile);
      fwrite((void *)q->barecall,1,13,tmpfile);
      fwrite((void *)q->call,1,14,tmpfile);
      fwrite((void *)q->locator,1,7,tmpfile);
      fwrite((void *)q->rst_RX,1,8,tmpfile);
      fwrite((void *)q->rst_TX,1,4,tmpfile);
      fwrite((void *)&q->points,1,2,tmpfile);
      fwrite((void *)&q->direction,1,2,tmpfile);
      q = q->next;
  
      paint_bar(cnt);
      cnt++;
   }
   fclose(tmpfile);

   error_log("write_contest_file()","Freeing window");
   drestore(save);
   sec_free(save);
  
   error_log("write_contest_file()","Renaming/Erasing");
   if(unlink(filename)) return(RENAME_ERR);
   if(rename(TMP_NAME,filename)) return(RENAME_ERR);

   return(OK);
};


// Seeks for a QSO-entry in the QSO-list
// Returns NULL if not found
struct qso *find_qso(pchar s)
{
   struct qso far *qsop;

   qsop = firstqso;
   while(qsop && strcmp(qsop->barecall,s)) qsop = qsop->next;
   return(qsop);
};


// Seeks for a specific locator in the QSO-list
// Return first locator found, NULL if none is found
struct qso far *find_locator(pchar s)
{
   struct qso far *qsop;

   qsop = firstqso;
   while(qsop && strcmp(qsop->locator,s)) qsop = qsop->next;
   return(qsop);
};


// Adds a QSO to the QSO-list
void add_qso(struct qso data)
{
   struct qso far *newqso;

   error_log("add_qso()","Allocating memory");
   newqso = sec_alloc(QSO_SIZE);

   *newqso = data;

   if(!firstqso)
   {
      firstqso = newqso;
      lastqso  = newqso;
      newqso->next = NULLqso;
      newqso->prev = NULLqso;
      return;
   }

   lastqso->next = newqso;
   newqso->prev = lastqso;
   newqso->next = NULLqso;
   lastqso = newqso;
   return;
};


// Seeks for first appearance of the locator s in the worked-square-list
// Returns NULL if not found
struct squareinfo *find_square(pchar s)
{
   struct squareinfo *hlp;
   uchar loc[7];

   hlp = firstsquare;
   strcpy(loc,s);
   loc[4] = 0;

   while(hlp)
   {
      if(!strcmp(hlp->name,loc)) return(hlp);
      hlp = hlp->next;
   }

   return(NULLsquare);
};

// Adds a locator to the worked-square-list
struct squareinfo *add_square(pchar s)
{
   struct squareinfo *hlp;
   struct squareinfo *new_square;

   error_log("add_square()","Allocating memory");
   new_square = sec_alloc(sizeof(struct squareinfo));

   new_square->worked = 0;
   new_square->points = 0;
   new_square->name[4] = 0;
   memcpy(new_square->name,s,4);

   hlp = firstsquare;
   while(hlp && strcmp(hlp->name,new_square->name) < 0) hlp = hlp->next;

   if(hlp)
   {
      new_square->prev = hlp->prev;
      if(hlp->prev) hlp->prev->next = new_square;
      else firstsquare = new_square;
      new_square->next = hlp;
      hlp->prev        = new_square;
   }
   else
   {
      new_square->prev = lastsquare;
      if(lastsquare) lastsquare->next = new_square;
      lastsquare = new_square;
      new_square->next = NULLsquare;
      if(!firstsquare) firstsquare = new_square;
   }

   return(new_square);
};


// Kills all QSOs from the QSO-list
void kill_all_qso()
{
   struct qso *qsop;
   struct qso *hlp;

   error_log("kill_all_qso()","Called");
   qsop = firstqso;
   while(qsop)
   {
      hlp = qsop->next;
      sec_free(qsop);
      qsop = hlp;
   }
   firstqso = NULLqso;
   lastqso = NULLqso;
};


// Kills the worked-square-list
void kill_all_squares()
{
   struct squareinfo *sqp;
   struct squareinfo *hlp;

   error_log("kill_all_squares()","Called");
   sqp = firstsquare;
   while(sqp)
   {
      hlp = sqp->next;
      sec_free(sqp);
      sqp = hlp;
   }
   firstsquare = NULLsquare;
   lastsquare = NULLsquare;
};

// Kills the prefix-list
void kill_all_prefixes()
{
   struct countryinfo far *ctp;
   struct countryinfo far *hlp;

   error_log("kill_all_prefixes()","Called");
   ctp = firstcountry;
   while(ctp)
   {
      hlp = ctp->next;
      sec_free(ctp->name);
      sec_free(ctp->name_eng);
      sec_free(ctp->valid_squares);
      sec_free(ctp->prefixes);
      sec_free(ctp);
      ctp = hlp;
   }
   firstcountry = NULLcountry;
   lastcountry = NULLcountry;
};

// Kills the station-database in memory
void kill_all_calls()
{
/*   switch(call_type)
   {
      case DISK:
         fclose(call_file);
         call_file = NULL;
         break;
      case EMS:
         ems_free(call_handle);
         call_handle = 0;
         break;
      case XMS:
         xms_free(call_handle);
         call_handle = 0;
         break;
   };*/
   call_cnt = 0;
};


// Ldt den Call-Index
unsigned char load_call_index()
{
/*   // Das File
   FILE *f;
   // Kompletter Dateiname
   char fname[128];

   struct call_index huge *hlp;
   unsigned rest;
   long offset;
   unsigned read; 
   unsigned size;  


   // Datei ffnen
   generate_path((char *)fn_path,(char *)"STATIONS.DEX",(char *)"\0",(char *)fname);
   if((f = fopen(fname,"rb")) == NULL) return(0);

   // Anzahl der Eintrge bestimmen
   fseek(f,0,SEEK_END);
   call_cnt = (ftell(f)-8)>>4;
   rest = call_cnt;

   fseek(f,8,SEEK_SET);

   call_type = NONE;
   if(xms_inst && use_xms)
   {
      size = ((long)((long)call_cnt*sizeof(struct call_index))+1023)>>10;
      if(xms_get_free()>=size)
      {
         call_handle = xms_alloc(size);
         call_type = XMS;

         hlp = lowalloc(512*sizeof(struct call_index));
         rest = call_cnt;
         offset = 0;
         while(rest)
         {
            read = fread(hlp,sizeof(struct call_index),rest>512?512:rest,f);
            xms_moveto(hlp,call_handle,offset,read*sizeof(struct call_index));
            offset+=(read*sizeof(struct call_index));
            rest-=read;
         };
         lowfree(hlp);
      };
   };

   if((call_type==NONE) && ems_inst && use_ems)
   {
      size = ((long)((long)call_cnt*sizeof(struct call_index))+0x3FFF)>>14;
      if(ems_get_free()>=size)
      {
         call_handle = ems_alloc(size);
         call_type = EMS;

         size = 0x4000/sizeof(struct call_index);
         rest = call_cnt;
         offset = 0;

         while(rest)
         {
            ems_setmap(call_handle,offset,0);   
            read = fread(ems_frame[0],sizeof(struct call_index),rest>size?size:rest,f);
            offset++;
            rest-=read;
         };
      };
   };

   if(call_type == NONE)
   {
      generate_path((char *)fn_path,(char *)"STATIONS.DEX",(char *)"\0",(char *)fname);
      if((call_file = fopen(fname,"rb+")) != NULL) call_type = DISK;
   };

   if(call_type == NONE)
   {
      fclose(f);
      return(0);
   };

   fclose(f);/*
   return(1);
};

// Ldt den Multi-Index in ein Memory-Handle, und gibt es
// zurck.
unsigned char load_multi_index()
{
/*   // Das File
   FILE *f;
   // Kompletter Dateiname
   char fname[128];
   // Swapbereich
   struct multi_index multi_ndx[32];
   unsigned i;

   // Datei ffnen
   *handle = 0;
   generate_path((char *)fn_path,(char *)"MULTIS.DEX",(char *)"\0",(char *)fname);
   if((f = fopen(fname,"rb")) == NULL) return;

   // Anzahl der Eintrge bestimmen
   fseek(f,0,SEEK_END);
   *cnt = (ftell(f)-8)/sizeof(struct multi_index);

   // Speicher allokieren
   *handle = xalloc((long)(*cnt)*sizeof(struct multi_index));

   // Einlesen
   fseek(f,8,SEEK_SET);
   i = *cnt;

   while(i)
   {
      fread(multi_ndx,i>32?32:i,sizeof(struct multi_index),f);
      xunlink(*handle,multi_ndx,(long)(*cnt-i)*sizeof(struct multi_index),
              (long)(i>32?32:i)*sizeof(struct multi_index));
      i-=(i>32?32:i);
   };

   fclose(f);*/
};


// Ldt den Locator-Index in ein Memory-Handle, und gibt es
// zurck.
unsigned char load_locator_index()
{
/*   // Das File
   FILE *f;
   // Kompletter Dateiname
   char fname[128];
   // Swapbereich
   struct loc_index loc_ndx[32];
   unsigned i;

   // Datei ffnen
   *handle = 0;
   generate_path(fn_path,"LOCATORS.DEX","\0",(char *)fname);
   if((f = fopen(fname,"rb")) == NULL) return;

   // Anzahl der Eintrge bestimmen
   fseek(f,0,SEEK_END);
   *cnt = (ftell(f)-8)/sizeof(struct loc_index);

   // Speicher allokieren
   *handle = xalloc((long)(*cnt)*sizeof(struct loc_index));

   // Einlesen
   fseek(f,8,SEEK_SET);
   i = *cnt;

   while(i)
   {
      fread(loc_ndx,i>32?32:i,sizeof(struct loc_index),f);
      xunlink(*handle,loc_ndx,(long)(*cnt-i)*sizeof(struct loc_index),
              (long)(i>32?32:i)*sizeof(struct loc_index));
      i-=(i>32?32:i);
   };

   fclose(f);*/
};


// Liest einen Calleintrag aus dem (bereits geffneten)
// File f, und liefert ihn in call_rec zurck.
void read_entry(FILE *f, struct callinfo *call_rec)
{
   // File-Variable fr das Info-File
   FILE *infof;

   // Kompletter Dateiname
   char fname[129];

   // Puffer fr einen DBF-Eintrag
   uchar buf[83];
   // Puffer fr den Infoeintrag zur Station
   uchar sbuf[80];


   fread(buf,1,83,f);   // DBF-Record einlesen.

   // Daten extrahieren
   call_rec->del = buf[0];
   memmove(call_rec->call,&buf[1],12);
   call_rec->call[12] = 0;

   memmove(sbuf,&buf[13],7);
   sbuf[7] = 0;
   call_rec->info_idx = atol(sbuf);

   memmove(call_rec->qrv_on,&buf[20],15);
   call_rec->qrv_on[15] = 0;

   memmove(call_rec->locator,&buf[35],34);
   call_rec->locator[34] = 0;

   memmove(sbuf,&buf[69],4);
   sbuf[4] = 0;
   call_rec->loc_date = atoi(sbuf);

   memmove(call_rec->multi,&buf[73],6);
   call_rec->multi[6] = 0;

   memmove(sbuf,&buf[79],4);
   sbuf[4] = 0;
   call_rec->multi_date = atoi(sbuf);

   // Infostring bearbeiten
   call_rec->info[0] = 0;
   if(call_rec->info_idx)
   {
      // Info-File ffnen und Eintrag einlesen.
      generate_path((char *)fn_path,(char *)"STATIONS.NFO",(char *)"\0",(char *)fname);
      if((infof = fopen(fname,"rb")) == NULL) return;

      fseek(infof,call_rec->info_idx,SEEK_SET);
      fread(sbuf,1,80,infof);
      strcpy(call_rec->info,sbuf);

      fclose(infof);
   };
};


// Looks up a call in the station-database
// Returns 0 if not found.
BOOL find_station(pchar call, struct callinfo far *call_rec)
{
/*   FILE *f;
   uchar fname[128];
   uchar s[16];

   unsigned left, right, pos;
   struct call_index pos_ind; 

   int result;

   unsigned calls_per_page;
   unsigned long offset;
   unsigned page;

   strcpy(s,call);
   s[15] = 0;

   result = 0;

   left = 0;
   right = call_cnt;
   pos = 0;


   while(pos!=((left+right)>>1))
   {
      pos = (left+right)>>1;

      switch(call_type)
      {
         case XMS:
            xms_movefrom(call_handle,(long)pos*sizeof(struct call_index),&pos_ind,sizeof(struct call_index));
            break;
         case EMS:
            calls_per_page = 0x4000/sizeof(struct call_index);
            page = pos/calls_per_page;
            offset = pos%calls_per_page;
            ems_setmap(call_handle,page,0);
            pos_ind = *((struct call_index far *)ems_frame[0]+offset);
            break;
         case DISK:
            fseek(call_file,((long)pos*sizeof(struct call_index))+8,SEEK_SET);
            fread(&pos_ind,1,sizeof(struct call_index),call_file);
            break;
      };

      result = strcmp(s,pos_ind.call);
      if(!result)
      {
         left = pos;
         right = pos;
      };

      if(result<0) right = pos;
      if(result>0) left = pos;
   };

   if(strcmp(s,pos_ind.call)) return(0);

   generate_path((char *)fn_path,(char *)"STATIONS.DBF",(char *)"\0",(char *)fname);
   if((f = fopen(fname,"rb")) == NULL) return(0);
   fseek(f,pos_ind.pos,SEEK_SET);

   read_entry(f,call_rec);

   fclose(f);
*/
//   return(1);
return(0);
};

// Looks up a call in the prefix-list
// Returns NULL if not found
struct countryinfo *find_prefix(pchar call)
{
   uchar s[15];
   uchar slen;
   struct countryinfo *hlp;
   pchar cp;

   strcpy(s," ");
   strcat(s,call);
   strcat(s," ");
   slen = strlen(s);

   while(slen>2)
   {
      hlp = firstcountry;
      while(hlp)
      {
         if(strstr((pchar)hlp->prefixes,(pchar)s)) return(hlp);
         hlp = hlp ->next;
      }
      slen--;
      s[slen] = 0;
      s[slen-1] = ' ';
   };

   return(NULLcountry);
};

// Loads the prefix-file
void load_prefixes()
{
   FILE *prefix_file;
   struct countryinfo new_prefix;
   struct countryinfo *new_p;
   uchar s[255];
   uchar hp[2048];
   pchar cp;
   pchar sp;
   void *save;

   error_log("load_prefixes()","Called");
   save = make_win(10,5,70,10,menucol);

   dprint(31,6,menucol,"read prefix-definitions:");

   generate_path((char *)fn_path,(char *)PREFIXES_NAME,(char *)"\0",(char *)s);
   if((prefix_file = fopen(s,"r")) == NULLfile)
   {
      dprint(33,8,menucol,"  not found!  ");
      drestore(save);
      sec_free(save);
      return;
   }

   new_prefix.worked = 0;
   new_prefix.points = 0;
   new_prefix.name_eng = NULL;
   new_prefix.valid_squares = NULL;
   new_prefix.name = NULL;
   new_prefix.prefixes = NULL;

   while(!feof(prefix_file))
   {
      fgets(s,255,(FILE *)prefix_file);
      sp = s;
      while(*sp==' ') sp++;
      if(*sp!=';')
      {
         strrev(sp);
         while((*sp==' ')||(*sp=='\n')) sp++;
         strrev(sp);
         strcpy(s,sp);
         if(!strcmpi(s,"END."))
         {
            new_p = sec_alloc(sizeof(struct countryinfo));
            new_p->worked = new_prefix.worked;
            new_prefix.worked = 0;
            new_p->points = new_prefix.points;
            new_prefix.points = 0;
            new_p->name_eng = new_prefix.name_eng;
            new_prefix.name_eng = NULL;
            new_p->name = new_prefix.name;
            new_prefix.name = NULL;
            new_p->valid_squares = new_prefix.valid_squares;
            new_prefix.valid_squares = NULL;
            new_p->prefixes = new_prefix.prefixes;
            new_prefix.prefixes = NULL;
               
            // Neuen Record einfgen
            if(!firstcountry)
            {
               firstcountry = new_p;
               lastcountry  = new_p;
               new_p->next = NULLcountry;
               new_p->prev = NULLcountry;
            }
            else
            {
               lastcountry->next = new_p;
               new_p->prev = lastcountry;
               lastcountry = new_p;
               new_p->next = NULLcountry;
            };
         }
         else
         {
            if((cp = strchr(s,'='))!=0)
            {
               *cp = 0;
               cp++;
               while(*cp==' ') cp++;
 
               strupr(s);
               strrev(s);
               sp = s;
               while(*sp==' ') sp++;
               strrev(sp);
               strcpy(s,sp);

               if(!strcmp(s,"NAME"))
               {
                  if(!new_prefix.name)
                  {
                     new_prefix.name = sec_alloc(strlen(cp)+1);
                     strcpy(new_prefix.name,cp);
                  };
               };
 
               if(!strcmp(s,"NAME_ENG"))
               {
                  if(!new_prefix.name_eng)
                  {
                     new_prefix.name_eng = sec_alloc(strlen(cp)+1);
                     strcpy(new_prefix.name_eng,cp);
                  };
               };

               if(!strcmp(s,"PREFIXES"))
               {
                  if(new_prefix.prefixes)
                  {
                     strcpy(hp,new_prefix.prefixes);
                     sec_free(new_prefix.prefixes);
                  }
                  else
                  {
                     strcpy(hp," ");
                  };
                  strcat(hp,strupr(cp));
                  strcat(hp," ");
                  new_prefix.prefixes = sec_alloc(strlen(hp)+1);
                  strcpy(new_prefix.prefixes,hp);
               };

               if(!strcmp(s,"SQUARES"))
               {
                  if(new_prefix.valid_squares)
                  {
                     strcpy(hp,new_prefix.valid_squares);
                     sec_free(new_prefix.valid_squares);
                  }
                  else
                  {
                     strcpy(hp," ");
                  };
                  strcat(hp,strupr(cp));
                  strcat(hp," ");
                  new_prefix.valid_squares = sec_alloc(strlen(hp)+1);
                  strcpy(new_prefix.valid_squares,hp);
               };
            };
         };
      };
   }

   drestore(save);
   sec_free(save);
   //dprintf(25,lastline,titlecol,"%3u Prfixe gelesen!",cnt-1);

   fclose(prefix_file);
};


// berprft, ob Call und Locator zusammenpassen
char check_call_and_locator(pchar callsign, pchar loc)
{
   uchar call[14];
   uchar add[4];
   uchar pref[6];
   uchar s[7];
   struct countryinfo *country;

   splitcall(callsign,pref,call,add);
   if(pref[0]) country = find_prefix(pref);
   else country = find_prefix(call);

   if(!country) return(CHK_UNKNOWN_PREFIX);
   else
   {
      strcpy(s,loc);
      s[4] = 0;
      if(country->valid_squares && !strstr(country->valid_squares,s)) return(CHK_WRONG_SQUARE);
   };
   return(CHK_OK);
};



