// dvideo.h - Definitions for direct video-RAM-access
// Written by Stefan Pfeiffer, DL1ELY, 940129
// Last modification: 960105
// (C) 1994 by Stefan Pfeiffer, DL1ELY@DB0PKE.#NRW.DEU.EU

// 940902: Huge bug fixed reported by OK1HX:
//   No video-output made on hercules-cards due to wrong
//   video-segment...mea culpa :(


#ifndef DVIDEO_H

#define DVIDEO_H

// colourcodes 
// Fr CONIO.H:
#define __COLORS

#define BLACK        0x00
#define BLUE         0x01
#define GREEN        0x02
#define CYAN         0x03
#define RED          0x04
#define MAGENTA      0x05
#define BROWN        0x06
#define LIGHTGRAY    0x07

#define DARKGRAY     0x08
#define LIGHTBLUE    0x09
#define LIGHTGREEN   0x0A
#define LIGHTCYAN    0x0B
#define LIGHTRED     0x0C
#define LIGHTMAGENTA 0x0D
#define YELLOW       0x0E
#define WHITE        0x0F

// Attributes for monochrome cards
#define BLACK_WHITE  0x07
#define WHITE_BLACK  0x70
#define UNDERLINED   0x01


// Macro for generating the color-byte
#define COL(fore,back)  (back<<4)+fore


#include "secmem.h"

// starting segment of video-RAM (0xB800 on color-cards,
// 0xB000 on monochrome ones)
extern unsigned int startseg;

// screenlines - 1
extern char lastline;

// screenrows-1
extern char lastrow;

// Old video-mode
extern char save_mode;


// Initializes the video-output.
// Has to be called before any other routine
void dstart(void);

// Deinitializes the video-output.
// Has to be called before exiting the programme
void dend(void);

// Displays a string at a specific position with a specific attribute
void dprint(char, char, unsigned char, unsigned  char *);

// Displays a formatted string the same as dprint
void dprintf(char, char, unsigned char, const unsigned char *, ...);

// Fills a part of the screen with a character
void dfill(char, char, char, char, unsigned char, unsigned char);

// Clears the whole screen with attr
void dclrscr(unsigned char);

// Moves a specific portion of the screen to another position
void dmove(char, char, char, char, char, char);

// Changes the attribute-bytes of a specific area
void dcolor(char, char, char, char, unsigned char);

// Saves a specific portion of the screen to memory
void *dsave(char, char, char, char);

// Calculates the size of an area of the screen
unsigned dcalcsize(char, char, char, char);

// Restores a portion of the screen
void drestore(char *);

// Paints a frame
void dframe(char, char, char, char, char);

#endif
