
/*
   locator.c - Original von DL5FBD in Pascal
   Written by Stefan Pfeiffer, DL1ELY, 940214
   Last Modification: 950328
   (C) 1994 by Stefan Pfeiffer, DL1ELY
*/

#include "locator.h"


/*
    Umrechnung eines Locators im WW-Format in relative Grofeld-
    Koordinaten zum Feld JO00.
    Positiv: stlich, Sdlich / Negativ: Westlich, Nrdlich
*/
void loc2rel(char *loc,int *X, int *Y)
{
   *X = ((*loc - 'J') * 10) + (*(loc+2) - '0');
   *Y = ((-(*(loc+1) - 'O')) * 10) - (*(loc+3) - '0');
   return;
};

/* Umrechnung von relativen Koordinaten in Grossfelder */
void rel2loc(int X, int Y, char *loc)
{
   if(X>=0)
   {
      *loc = (X/10)+'J';
      *(loc+2) = '0'+(X%10);
   }
   else
   {
      *loc = 'I'-(abs(X)/10);
      if(abs(X)%10 == 0) *(loc+2) = '0';
         else *(loc+2) = ':'-(abs(X)%10);
   };

   if(Y>0)
   {
      *(loc+1) = 'N' - ((Y-1)/10);
      *(loc+3) = '9' - ((Y-1)%10);
   }
   else
   {
      *(loc+1) = 'O' + (abs(Y)/10);
      *(loc+3) = '0' + (abs(Y)%10);
   };
   *(loc+4) = 0;
};
  
/* Berechnung des Sinus zu einem gegebenen Gradwinkel */
double gsin(double winkel)
{
   return(sin(winkel*PI/180));
};


/* Berechnung des Cosinus zu einem gegebenen Gradwinkel    */
double gcos(double winkel)
{
   return(cos(winkel*PI/180));
};


/* Berechnung des Gradwinkels zum gegebenen Cosinuswert    */
double arcgcos(double cosinus)
{
   double arcbog; /* Hilfsvariable vor Gradumrechnung */
  
   if(cosinus>=1) return(0);  /* Sonderfall   0 Grad */
   else
      if(cosinus<=-1) return(180);  /* Sonderfall 180 Grad */
      else
      {
         arcbog = PI/2-atan(cosinus/(sqrt(1-(cosinus*cosinus))));
         /* Umrechnung vom Bogenma in Grad */
         return(arcbog*180/PI);
      };
};


/*
   dist_and_dir()                             
   Die Prozedur dient zur Berechnung von Entfernung und      
   Richtung bei gegebenen geografischen Koordinaten im       
   Gradmass.                                                 
   Ergebnis sind Entfernung in Kilometern und Richtung in    
   Grad von QTH1 nach QTH2.                                  
   O1,N1 Oestliche Laenge,Noerdliche Breite von QTH1         
   O2,N2 Oestliche Laenge,Noerdliche Breite von QTH2         
*/

void dist_and_dir(double o1, double n1, double o2, double n2, double *dist, double *dir)
{
   double EW,RV;  /* EW Entfernungswinkel */
                  /* RV vorlaeufige Richtung */

   /* Beginn der eigentlichen Entfernungs-Richtungsberechnung */

   if((o1==o2) && (n1==n2))
   {
      *dir = 361;
      *dist = 3;
      return;
   };

   EW = arcgcos(gsin(n1)*gsin(n2)+gcos(n1)*gcos(n2)*gcos(o2-o1));
   *dist = 111.2*EW;

   /* Richtungsberechnung */
   RV = arcgcos((gsin(n2)-gsin(n1)*gcos(EW))/(gcos(n1)*gsin(EW)));
   if(gsin(o2-o1)>=0) *dir = RV;
   if(gsin(o2-o1)< 0) *dir = 360-RV;
};


/*
   char valid_locator()                                   
   Diese FUNCTION dient zur Pruefung ob der uebergebene    
   QTH-Kenner ein korrektes Format hat. 
   KORREKT in diesem Falle:
   Alles zwischen HM00AA bis LQ99XX (also REICHLICH Europa, hi).                   
*/

char valid_locator(char *qthkenn)
{
   return((!*(qthkenn+6)) &&
          ((toupper(*qthkenn)>='H')&&(toupper(*qthkenn)<='L')) &&
          ((toupper(*(qthkenn+1))>='M')&&(toupper(*(qthkenn+1))<='Q')) &&
          (isdigit(*(qthkenn+2))) &&
          (isdigit(*(qthkenn+3))) &&
          ((toupper(*(qthkenn+4))>='A')&&(toupper(*(qthkenn+4))<='X')) &&
          ((toupper(*(qthkenn+5))>='A')&&(toupper(*(qthkenn+5))<='X')));
}         
         


/* 
   void loc2coord()                                 
   Diese Procedure dient zum Umwandeln eines neuen QTH-    
   kenners in geografische Laenge und Breite               
*/
void loc2coord(char *qthkenn, double *oeslae, double *noebre)
{
   unsigned char asckor[6] = 
      { 65, 65, 48, 48, 65, 65 };
   unsigned char maske[6] = 
      { 223, 223, 255, 255, 223, 223 };
   char i;
   unsigned char wiinfo[6];


   /* Ermittlung der Indexziffern aus dem QTH-Kenner */
   for(i=0;i<6;i++)
   {
      wiinfo[i] = (*(qthkenn+i) & maske[i]) - asckor[i];
   };

   /* Berechnung der geografischen Koordinate aus den Indexziffern */
   *oeslae = -180+(double)wiinfo[0]*20+(double)wiinfo[2]*2+(double)wiinfo[4]/12+1/24;
   *noebre = -90+(double)wiinfo[1]*10+(double)wiinfo[3]*1+(double)wiinfo[5]/24+1/48;
};

// Wandelt einen Locator in Groschrift um. Kann man an sich auch genausogut
// mit strupr machen, hi.
void upcase_loc(char *loc)
{
   *(loc) = toupper(*(loc));
   *(loc+1) = toupper(*(loc+1));
   *(loc+4) = toupper(*(loc+4));
   *(loc+5) = toupper(*(loc+5));
}

// Hat der Locator uerlich die Form eines Locators?
// HIER ist auch AA00ZZ erlaubt, obwohl dies KEIN gltiger Locator ist!
char loc_shape(char *loc)
{
   return(!*(loc+6)&&
          isalpha(*loc)&&isalpha(*(loc+1))&&
          isdigit(*(loc+2))&&isdigit(*(loc+3))&&
          isalpha(*(loc+4))&&isalpha(*(loc+5)));
};

// Ist der String fr FiveNine fltiger Locator?
// Er mu mindestens 3 Zeichen lang sein, und alle vorhandenen Stellen
// mssen dem Locator-Format (von hinten) entsprechen.
// In FiveNine werden die fehlenden ersten Stellen aus dem eigenen
// Locator ergnzt.
// Beispiel: 1IO, 31IO, O31IO, JO31IO liefert 1...
char is_loc(char *loc)
{
   char *cp;
   unsigned loclen;

   loclen = strlen(loc);
   if(loclen>6 || loclen<3) return(0);

   if(loclen == 6) return(valid_locator(loc));

   cp = loc+loclen-1;
   if((*cp<'A') || (*cp>'X')) return(0);
   cp--;
   if((*cp<'A') || (*cp>'X')) return(0);
   cp--;
   if(!isdigit(*cp)) return(0);

   if(loclen>3)
   {
      cp--;
      if(!isdigit(*cp)) return(0);
   };
   if(loclen>4)
   {
      cp--;
      if((*cp<'M') || (*cp>'Q')) return(0);
   };

   if(loclen>5)
   {
      cp--;
      if((*cp<'H') || (*cp>'L')) return(0);
   };

   return(1);
};

