// map.c - graphical map of europe 
// Written By Stefan Pfeiffer, DL1ELY@DB0PKE, 940713 
// Last modification: 960111
// (C) 1994 by Stefan Pfeiffer, DL1ELY@DB0PKE 

#include <graphics.h>
#include "map.h"
#include "global.h"
#include "input.h"
#include "egavga.h"
#include "dvideo.h"
#include "locator.h"
#include "tools.h"
#include "fivenine.h"
#include <stdio.h>
#include <string.h>


extern uchar fn_path[128];
//extern unsigned char use_ems;
//extern unsigned char use_xms;

// Speicher fr Kartendaten
unsigned *map = NULL;

unsigned *border = NULL;

unsigned map_size = 0;
unsigned border_size = 0;

// height and width of map in 1/1000th degree
unsigned map_x, map_y;
// left and upper border of map in degrees
double map_left;
double map_top;

// Scaling-factor for map
double scale;

// Saved screen before switching to graphics mode
void *map_save;

// Colors of map
unsigned char mapcol;
unsigned char bordercol;
unsigned char maptextcol;

unsigned char gridcol;
unsigned char gridinvcol;


// Gibt den Speicherbereich fr die Karte frei
void free_map_data()
{
   error_log("free_map()","Freeing Map-Data");
   
   sec_free(map);
   map = NULL;

   sec_free(border);
   border = NULL;

   error_log("free_map()","Map-Data OK");
};


void load_map_data()
{
   error_log("load_map_data()","Loading Map");
   load_map();
   error_log("load_map_data()","Map OK");
   error_log("load_map_data()","Loading Borders");
   load_borders();
   error_log("load_map_data()","Borders OK");
};

  
// Erstellt aus dem File map_file das Schnell-Lade-File fst_file...
unsigned make_fst_map()
{
   FILE *inf, *outf;
   unsigned int *buf;
   unsigned read;
   unsigned size, bytes;
   unsigned i;
   unsigned processed = 0;
   void *save;
   char hlp[128], hlp2[128];

 
   generate_path((char *)fn_path,(char *)MAP_NAME,(char *)"\0",(char *)hlp);
   error_log("make_fst_map()","Opening %s",hlp);
   if((inf = fopen(hlp,"rb")) == NULL)
   {
      error_log("make_fst_map()","File not found");
      return(0);
   };
   generate_path((char *)fn_path,(char *)MAP_FST_NAME,(char *)"\0",(char *)hlp2);
   error_log("make_fst_map()","Creating %s",hlp2);
   outf = fopen(hlp2,"wb");

   error_log("make_fst_map()","Allocating buffer (0x1000 byte)");
   buf = sec_alloc(0x1000);

   error_log("make_fst_map()","Buffer OK");

   // Get the size of the map (25 bytes header)
   fseek(inf,0,SEEK_END);
   size = ftell(inf)-25;
   error_log("make_fst_map()","Size of %s is %u byte",hlp,size);
  
   error_log("make_fst_map()","Reading header");
   // Header lesen
   fseek(inf,5,SEEK_SET);
   fread((void *)&map_left,1,8,inf);
   fread((void *)&map_top,1,8,inf);
   fread((void *)&map_x,1,2,inf);
   fread((void *)&map_y,1,2,inf);
  
   scale = (double)xmax/map_x;
   if(scale>(double)(ymax/map_y)) scale = (double)ymax/map_y;

   error_log("make_fst_map()","Writing header %s",hlp2);
   fwrite((void *)&xmax,1,2,outf);
   fwrite((void *)&ymax,1,2,outf);
   fwrite((void *)&scale,8,1,outf);
   fwrite((void *)&map_left,8,1,outf);
   fwrite((void *)&map_top,8,1,outf);
   fwrite((void *)&map_x,1,2,outf);
   fwrite((void *)&map_y,1,2,outf);
  
   bytes = size;

   error_log("make_fst_map()","Opening window");
   save = make_win(10,5,70,10,menucol);
   dprint(34,6,menucol," Create FST ");

   init_bar(9,menucol,size);

   error_log("make_fst_map()","Starting conversion");
   while(bytes)
   {
      error_log("make_fst_map()","Reading 0x1000 byte");
      read = fread(buf,1,0x1000,inf);
      bytes -= read;
      error_log("make_fst_map()","Read: %d",read);
    
      for(i=0;i<read/2;i++)
      {  
         if(buf[i]!=0xFFFF) buf[i] = (int)(buf[i]*scale);
         processed+=2;
         paint_bar(processed);
      };
      error_log("make_fst_map()","Conversion of buffer OK");
      fwrite(buf,1,read,outf);
      error_log("make_fst_map()","Buffer written to %s",hlp2);
   };

   error_log("make_fst_map()","Freeing window");
   drestore(save);
   sec_free(save);

   error_log("make_fst_map()","Closing files");
   fclose(inf);
   fclose(outf);
   error_log("make_fst_map()","Freeing buffer");
   sec_free(buf);  
  
   return(1);
};


// Erstellt aus dem File map_file das Schnell-Lade-File fst_file...
unsigned make_fst_border()
{
   FILE *inf, *outf;
   unsigned int *buf;
   unsigned read;
   unsigned size, bytes;
   unsigned i;
   double m_left, m_top;
   unsigned m_y, m_x;
   unsigned processed = 0;
   void *save;
   char hlp[128], hlp2[128];
  
   generate_path((char *)fn_path,(char *)BORDER_NAME,(char *)"\0",(char *)hlp);
   error_log("make_fst_border()","Searching %s",hlp);

   if((inf = fopen(hlp,"rb")) == NULL)
   {
      error_log("make_fst_border()","File not found!");
      return(0);
   };

   generate_path((char *)fn_path,(char *)BORDER_FST_NAME,(char *)"\0",(char *)hlp2);
   error_log("make_fst_border()","Creating %s",hlp2);
   outf = fopen(hlp2,"wb");
 
   error_log("make_fst_border()","Allocating buffer (0x1000 byte)");
   buf = sec_alloc(0x1000);

   // Get the size of the map (25 bytes header)
   fseek(inf,0,SEEK_END);
   size = ftell(inf)-25;
   error_log("make_fst_border()","Size of %s: %u byte",hlp,size);
  
   error_log("make_fst_border()","Reading header");
   // Header lesen
   fseek(inf,5,SEEK_SET);
   fread((void *)&m_left,1,8,inf);
   fread((void *)&m_top,1,8,inf);
   fread((void *)&m_x,1,2,inf);
   fread((void *)&m_y,1,2,inf);

   error_log("make_fst_border","Copying header to %s",hlp2);
   fwrite((void *)&xmax,1,2,outf);
   fwrite((void *)&ymax,1,2,outf);
   fwrite((void *)&scale,8,1,outf);
   fwrite((void *)&map_left,8,1,outf);
   fwrite((void *)&map_top,8,1,outf);
   fwrite((void *)&map_x,1,2,outf);
   fwrite((void *)&map_y,1,2,outf);
  
   bytes = size;

   error_log("make_fst_border()","Opening window");
   save = make_win(10,5,70,10,menucol);
   dprint(34,6,menucol," Create FST ");

   init_bar(9,menucol,size);

   error_log("make_fst_border()","Starting Conversion");
   while(bytes)
   {
      error_log("make_fst_border()","Reading 0x1000 byte from %s",hlp);
      read = fread(buf,1,0x1000,inf);
      bytes -= read;
      error_log("make_fst_border()","Read: %d",read);
    
      error_log("make_fst_border()","Processing buffer");
      for(i=0;i<read/2;i++)
      {  
         if(buf[i]!=0xFFFF) buf[i] = (int)(buf[i]*scale);
         processed+=2;
         paint_bar(processed);
      };  
      error_log("make_fst_border()","Buffer OK");
      error_log("make_fst_border()","Writing buffer to %s",hlp2);
      fwrite(buf,1,read,outf);
   };
   error_log("make_fst_border()","Conversion ok");

   error_log("make_fst_border()","Freeing window");
   drestore(save);
   sec_free(save);

   error_log("make_fst_border()","Closing files");
   fclose(inf);
   fclose(outf);
   error_log("make_fst_border()","Freeing buffer");
   sec_free(buf);  
  
   return(1);
};
 
 
// Load the borders from a file
void load_borders()
{
   FILE *f;
   unsigned read;
   unsigned x, y;
   unsigned size;
   unsigned offset;
   char hlp[128];

   error_log("load_borders()","Initializing map");
   if(!graphics) return;

   // Wenn kein FST-File vorhanden ist -> Erstellen und Laden
   generate_path((char *)fn_path,(char *)BORDER_FST_NAME,(char *)"\0",(char *)hlp);
   error_log("load_borders()","Searching %s",hlp);
   if((f = fopen(hlp,"rb")) == NULL)
   {
      error_log("load_borders()","File not found");
      if(make_fst_border()) load_borders();
      return;
   };
    
   error_log("load_borders()","Reading %s",hlp);
   fread((void *)&x,1,2,f);
   fread((void *)&y,1,2,f);

   if((x!=xmax)||(y!=ymax))
   {
      error_log("load_borders()","Resolution does not match");
      fclose(f);
      if(make_fst_border()) load_borders();
      return;
   };
    
   fseek(f,0,SEEK_END);
   border_size = ftell(f)-32;
   error_log("load_borders()","Size of %s is %u byte",hlp,border_size);

   error_log("load_borders()","Reading %s",hlp);
   // First 32 bytes are obsolete
   fseek(f,32,SEEK_SET);
    
   border = sec_alloc(border_size);
      
   fread(border,1,border_size,f);

   fclose(f);
   return;
};
    
  
  
// Load the map from a file
void load_map()
{
   FILE *f;
   unsigned x, y;
   unsigned size;
   char hlp[128];


   error_log("load_map()","Initializing map");
   if(!graphics) return;

   generate_path(fn_path,MAP_FST_NAME,"",hlp);
   error_log("load_map()","Searching %s",hlp);
   if((f = fopen(hlp,"rb")) == NULL)
   {
      error_log("load_map()","File not found");
      if(make_fst_map()) load_map();
      return;
   };
  
   error_log("load_map()","Reading %s",hlp);
   fread((void *)&x,1,2,f);
   fread((void *)&y,1,2,f);

   if((x!=xmax)||(y!=ymax))
   {
      error_log("load_map()","Resolution does not match");
      fclose(f);
      if(make_fst_map()) load_map();  
      return;
   };
    
   fseek(f,0,SEEK_END);
   map_size = ftell(f)-32;
   error_log("load_map()","Size of %s is %u byte",hlp,map_size);

   error_log("load_map()","Reading header");
   fseek(f,4,SEEK_SET);
   fread((void *)&scale,8,1,f);
   fread((void *)&map_left,8,1,f);
   fread((void *)&map_top,8,1,f);
   fread((void *)&map_x,1,2,f);
   fread((void *)&map_y,1,2,f);
   error_log("load_map()","Header OK");

   // First 32 bytes are obsolete
   fseek(f,32,SEEK_SET);
  

   map = sec_alloc(map_size);
      
   fread(map,1,map_size,f);

   fclose(f);
   return;
};
    
// Displays the borders
void display_borders()
{
   unsigned x,y;
   char firstpoint = 1;
   unsigned *buf;
   unsigned rest;
   unsigned pos;


   setcolor(bordercol);

   buf = border;
   pos = 0;

   y = 0xFFFF;

   rest = border_size>>1;
   
   while(rest)
   {   
      if(y == 0xFFFF)
      {
         x = buf[pos];
         if(x == 0xFFFF) firstpoint = 1;
         else y = 0;
      }
      else
      {
         y = buf[pos];
         if(firstpoint)
         {
            moveto(x,y);
            firstpoint = 0;
         }
         else lineto(x,y);
         y = 0xFFFF;
      };
      
      pos++;
      rest--;
   };
};


// Displays the map after having switched to graphics mode
void display_map(char *call, char *loc)
{
   unsigned x,y;
   char firstpoint = 1;
   char s[20];
   unsigned abschnitt;
   unsigned *buf;
   unsigned rest;
   unsigned pos;


   setcolor(mapcol);
   rectangle(0,0,(int)(map_x*scale),(int)(map_y*scale));
  
   rest = map_size>>1;
   pos = 0;
   y = 0xFFFF;
   buf = map;

   while(rest)
   {
      if(y == 0xFFFF)
      {
         x = buf[pos];
         if(x == 0xFFFF) firstpoint = 1;
         else y = 0;
      }
      else
      {
         y = buf[pos];
         if(firstpoint)
         {
            moveto(x,y);
            firstpoint = 0;
         }
         else lineto(x,y);
         y = 0xFFFF;
      };
      pos++;
      rest--;
   };

   x = (int)(-map_left*scale*1000.0);
   y = (int)((map_top-63)*scale*1000.0);

   setfillstyle(SOLID_FILL,BLACK);
   bar(1,1,x-2,y-1);
   setcolor(maptextcol);
   line(1,y,x-2,y);
   line(1,y-1,x-2,y-1);
   line(x-1,1,x-1,y);
   line(x,1,x,y);
   line(x+1,1,x+1,y);

   x>>=1;
   settextjustify(LEFT_TEXT,CENTER_TEXT);
   abschnitt = y/3;

   sprintf(s,"%s %s",call,loc);
   outtextxy(x-(textwidth(s)>>1),abschnitt,s);

   strcpy(s,"p = Drucken");
   outtextxy(x-(textwidth(s)>>1),2*abschnitt+1,s);
};
  

void display_loc(char *loc, char *call)
{
   double north;
   double east;
  
   unsigned x,y;
 
 
   if(!*loc) return;
  
   loc2coord(loc,&east,&north);
   x = (int)((east-map_left)*1000*scale);
   y = (int)((map_top-north)*1000*scale);
   setcolor(maptextcol);
   line(x,y-4,x,y+4);
   line(x-4,y,x+4,y);
   if(*call)
   {
      y-=13;
      x-=textwidth(call)/2;
      outtextxy(x,y,call);
   };
};

void text_mode()
{
   restorecrtmode();
   closegraph();
  
   switch(graphics)
   {
      case EGA:
         ega_setchars();   
         set_blink(NO_BLINK);
         def_cursor(0,7);
         break;
      case VGA:
         vga_setchars();
         set_blink(NO_BLINK);
         def_cursor(0,7);
         break;
   };
  
   drestore(map_save);
   sec_free(map_save);
};


int graph_mode()
{
   int gd,gm;
   char *windir;
  
   if(!graphics)
   {
      ok_box("Warning","No EGA- or VGA-adaptor found!");
      return(0);
   };
    
   map_save = dsave(0,0,79,lastline);

   gd = DETECT;
   initgraph(&gd,&gm,"");
   if(graphresult())
   {
      drestore(map_save);
      sec_free(map_save);
      ok_box("Error","Graphics driver not found!");
      return(0);
   };
  
   if((xmax==799) && (ymax==599) && (is_windows())) 
   {
      setfillstyle(SOLID_FILL,BLACK);
      bar(0,300,xmax,ymax);
   };
   return(1);
};


void display_path(char *loc1, char *loc2, char *call)
{
   double n1,e1,n2,e2;
   signed x1,y1,x2,y2;
   double dist, dir;
  
   if(*loc1) loc2coord(loc1,&e1,&n1);
   loc2coord(loc2,&e2,&n2);
  
   x1 = (int)((e1-map_left)*1000*scale);
   y1 = (int)((map_top-n1)*1000*scale);
  
   x2 = (int)((e2-map_left)*1000*scale);
   y2 = (int)((map_top-n2)*1000*scale);
  
   setcolor(maptextcol);
  
   if(strcmp(loc1,loc2))
   {
      if(*loc1)
      {
         dist_and_dir(e1,n1,e2,n2,&dist,&dir);
         line(x1,y1,x2,y2);
      }
      else dir = 0;
     
      display_loc(loc2,"\0");
      
      if((dir>315)||(dir<=45))
      {
         x1 = x2-(textwidth(call)/2);
         y1 = y2-13;
      };
      if((dir>45)&&(dir<=135))
      {
         x1 = x2+7;
         y1 = y2-3;
      };
      if((dir>135)&&(dir<=225))
      {
         x1 = x2-(textwidth(call)/2);
         y1 = y2+8;
      };
      if((dir>225)&&(dir<=315))
      {
         x1 = x2-textwidth(call)-7;
         y1 = y2-3;
      };
    
      if(x1 < 0)
      {
         x1 = 0;
         if((dir>225)&&(dir<=325)) y1 = y1-10;
      };  
    
      if(x1+textwidth(call) > xmax)
      {
         x1 = xmax-textwidth(call)-3;
         if((dir>45)&&(dir<=135)) y1 = y1-10;
      };  
    
      outtextxy(x1,y1,call);        
   };
};


void display_grid()
{
   int i;
   unsigned x, y;
  
  
   i = (map_left/2)*2;
   y = map_y*scale;
   while(i<map_left+(map_x/1000))
   {
      x = (int)(((i-map_left)*scale*1000.0)+0.5);
      if(i%20) setcolor(gridcol);
      else
      {
         setcolor(gridinvcol);
         line(x-1,0,x-1,y);
      };
      line(x,0,x,y);
      line(x+1,0,x+1,y);
      i+=2;
   };  

   i = (int)map_top;
   x = map_x*scale;
   while(i>=map_top-(map_y/1000))
   {
      y = (int)(((map_top-i)*scale*1000.0)+0.5);
      if(i%10) setcolor(gridcol);
      else
      {
         setcolor(gridinvcol);
         line(0,y+1,x,y+1);
      };
      line(0,y,x,y);
      i--;
   };  
};
        
