/* mapcomp.c - Kartencompiler fr WW-Locator */
/* Written by Stefan Pfeiffer, DL1ELY@DB0PKE, 940712 */
/* Last Modification: 940807 */
/* (C) 1994 by Stefan Pfeiffer, DL1ELY@DB0PKE */

#define PRGNAME "Map-Compiler"
#define VERSION "v1.11"
#define AUTHOR "Stefan Pfeiffer, DL1ELY@DB0PKE"
#define DATE "940807"


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <alloc.h>
#include <math.h>

struct grafln
{
    unsigned int x1,y1,x2,y2;
};

double x1,y1,x2,y2;
unsigned int lines=0;
struct grafln far *ln;

void doline(char *s)
{
    char *cp;

    cp = s;
    while(*cp==' ') cp++;
    s = cp;

    while(*cp!=' ') cp++;
    while(*cp==' ') cp++;
    x1 = atof(s);
    s = cp;
    while(*cp!=' ') cp++;
    while(*cp==' ') cp++;
    y1 = atof(s);
    s = cp;
    while(*cp!=' ') cp++;
    while(*cp==' ') cp++;
    x2 = atof(s);
    y2 = atof(cp);
};

unsigned int find_start_pt(void)
{
    unsigned int low_x, low_y;
    unsigned int i = 0;
    unsigned index = 0xFFFF;

    low_x = 0xFFFF;
    low_y = 0xFFFF;

    while(i<lines)
    {
        if((ln[i].x1)||(ln[i].y1))
        {
            if((ln[i].x1 < low_x)||((ln[i].x1 == low_x)&&(ln[i].y1<low_y)))
            {
                low_x = ln[i].x1;
                low_y = ln[i].y1;
                index = i;
            };
        };
        i++;
    };
    return(index);
};

unsigned int find_next_pt(unsigned int x, unsigned int y)
{
    unsigned int i = 0;

    while(i<lines)
    {
        if((ln[i].x1)||(ln[i].y1))
          if((abs(ln[i].x1 - x) < 10)&&(abs(ln[i].y1 - y) < 10)) return(i);
        i++;
    };
    return(0xFFFF);
};

int in_box(double x, double y, double x1, double y1, double x2, double y2)
{
    if((x>=x1)&&(x<=x2)&&(y>=y1)&&(y<=y2)) return(1);
    return(0);
};

int main(int argc, char **argv)
{
  FILE *map_file;
  FILE *src_file;
  char mapname[128];
  char input[1024];
  char *cp1;

  double left   = -10;
  double top    = 65;
  double right  = 30;
  double bottom = 35;

  double n = 49.75;
  double e = 12.25;

  unsigned int max_x = 0;
  unsigned int max_y = 0;
  unsigned int x;
  unsigned int index, index2;

  
  unsigned int points = 0;


  fprintf(stderr,"\n%s %s by %s, %s\n\n",PRGNAME,VERSION,AUTHOR,DATE);

  if((argc<2) || (argc > 3))
  {
    fprintf(stderr,"Usage:\n  MAPCOMP <srcfile> [mapfile[.MAP]]\n\n");
    fprintf(stderr,"If the mapfile is omitted, it will be called <srcfile>.MAP\n");
    return(0);
  };

  src_file = fopen(argv[1],"rt");
  if(!src_file)
  {
    fprintf(stderr,"File %s not found!\n",argv[1]);
    return(0);
  };

  if(!argv[2])
  {
    strcpy(mapname,argv[1]);
    cp1 = strchr(mapname,'.');
    if(*cp1) cp1 = 0;
    strcat(mapname,".MAP");
  }
  else
  {
    strcpy(mapname,argv[2]);
    cp1 = strchr(mapname,'.');
    if(!cp1) strcat(mapname,".MAP");
  };

  map_file = fopen(mapname,"wb");
  if(!map_file)
  {
    fprintf(stderr,"Can't create %s!",mapname);
    fclose(src_file);
    return(0);
  };

  fprintf(stderr,"Determining mapborders...\n");
  ln = farcalloc(20000,sizeof(struct grafln));
  if(!ln)
  {
      fprintf(stderr,"Zu wenig Speicher!\n");
      exit(1);
  };
  while(!feof(src_file))
  {
    fgets(input,1024,src_file);
    
    doline(input);

    if(in_box(x1,y1,left,bottom,right,top)&&
       in_box(x2,y2,left,bottom,right,top))
    {
      // Folgendes NUR, wenn die DL/Y-Grenze raus soll.
      if((!in_box(x1,y1,9,n,e,54))&&
         (!in_box(x2,y2,9,n,e,54)))
      {
        ln[lines].x1 = (int)((x1-left)*1000);
        ln[lines].y1 = (int)((top-y1)*1000);
        ln[lines].x2 = (int)((x2-left)*1000);
        ln[lines].y2 = (int)((top-y2)*1000);
        if(max_x<ln[lines].x1) max_x = ln[lines].x1;
        if(max_x<ln[lines].x2) max_x = ln[lines].x2;
        if(max_y<ln[lines].y1) max_y = ln[lines].y1;
        if(max_y<ln[lines].y2) max_y = ln[lines].y2;
        lines++;
      };
    };

    if(lines%100 == 0) putch('.');
  };

  putch('\n');
  putch('\r');

  fprintf(stderr,"Origin: %4.2f / %4.2f\n",left,top);

  fclose(src_file);

  fprintf(stderr,"Writing header...\n");
  strcpy(input,"MAP");
  fwrite(input,3,1,map_file);
  x = 0x0100;
  fwrite(&x,2,1,map_file);
  fwrite(&left,sizeof(left),1,map_file);
  fwrite(&top,sizeof(top),1,map_file);
  x = 0;
  fwrite(&x,2,1,map_file);
  fwrite(&x,2,1,map_file);

  fprintf(stderr,"Compiling map...\n");
  while((index = find_start_pt()) != 0xFFFF)
  {
    fwrite(&ln[index].x1,2,1,map_file);
    fwrite(&ln[index].y1,2,1,map_file);
    ln[index].x1 = 0;
    ln[index].y1 = 0;
    while((index2 = find_next_pt(ln[index].x2,ln[index].y2)) != 0xFFFF)
    {
      fwrite(&ln[index2].x1,2,1,map_file);
      fwrite(&ln[index2].y1,2,1,map_file);
      ln[index2].x1 = 0;
      ln[index2].y1 = 0;
      index = index2;
      points++;
    };
    fwrite(&ln[index].x2,2,1,map_file);
    fwrite(&ln[index].y2,2,1,map_file);
    points++;
    x = 0xFFFF;
    fwrite(&x,2,1,map_file);
    fprintf(stderr,"\r%d",points);
  };

  fseek(map_file,(2*sizeof(double))+5,SEEK_SET);
  fwrite(&max_x,2,1,map_file);
  fwrite(&max_y,2,1,map_file);
  
  fprintf(stderr,"\r%d points compiled.\n",points);

  fclose(map_file);
  farfree(ln);

  return(0);
};
        
       


