// MMANAGER.C - Functions to access EMS, XMS, UMB, HMA, DOS
// Written by Stefan Pfeiffer, DL1ELY, 941014
// Last modification: 950228
// (C) 1994 by Stefan Pfeiffer, DL1ELY


#include "mmanager.h"
#include "tools.h"


// Variablen fr DOS
unsigned char dos_umb_link;
unsigned char dos_mem_alloc;

// Variablen fr EMS
unsigned char ems_inst = 0;
void far *ems_frame[4];
unsigned char ems_ver_lo, ems_ver_hi;


// Variablen fr XMS
void far *xms_page;
unsigned char xms_page_type = NONE;

struct xms_cb block;

unsigned char xms_inst = 0;


void (far * xms_service)(void);

unsigned char xms_ver_hi, xms_ver_lo;
unsigned char xms_rev_hi, xms_rev_lo;
unsigned char xms_hma_found;



void mm_start(void)
{
  struct REGPACK r;
  unsigned int xms_ver;
  void far *ptr;

  error_log("mm_start()","Initializing...");
  if(_osmajor > 4)
  {
    error_log("mm_start()","Set UMB Link");
    r.r_ax = 0x5802;
    intr(DOS_INT,&r);
    dos_umb_link = r.r_bx&0xFF;
    
    r.r_ax = 0x5803;
    r.r_bx = 0x0001;
    intr(DOS_INT,&r);
  };
  
  if(_osmajor > 3)
  {
    error_log("mm_start()","Set memory allocation");
    r.r_ax = 0x5800;
    intr(DOS_INT,&r);
    dos_mem_alloc = r.r_bx&0xFF;
    
    r.r_ax = 0x5801;
    r.r_bx = _osmajor>4?USE_UMB|LAST_FIT:LAST_FIT;
    intr(DOS_INT,&r);
  };
  
  error_log("mm_start()","Looking fr XMS");
  r.r_ax = 0x4300;
  intr(XMS_INT,&r);
  xms_inst = (r.r_ax&0xFF) == 0x80;

  if(xms_inst)
  {
    error_log("mm_start()","XMS found");
    r.r_ax = 0x4310;
    intr(XMS_INT,&r);
    error_log("mm_start()","Getting XMS-Data");
    xms_service = MK_FP(r.r_es,r.r_bx);

    _AX = 0;
    xms_service();

    xms_ver = _AX;
    xms_ver_lo = xms_ver&0xFF;
    xms_ver_hi = xms_ver >> 8;

    xms_rev_hi = _BX >> 8;
    xms_rev_lo = _BX&0xFF;

    xms_hma_found = _DX & 0xFF;

    error_log("mm_start()","Allocating swaparea");
    xms_alloc_swaparea();
  };
  
  
  error_log("mm_start()","Looking for EMS");
  ptr = getvect(EMM_INT);
  ptr = MK_FP(FP_SEG(ptr),10);

  if(!_fstrncmp((char far *)ptr,(char far *)EMM_SIG,8)) ems_inst = 1;

  if(!ems_inst)
  {
    error_log("mm_start()","EMS not found");
    return;
  };
  
  error_log("mm_start()","Getting EMS-Data");
  r.r_ax = 0x4000;
  intr(EMM_INT,&r);
  ems_inst = (r.r_ax>>8 == 0);

  if(ems_inst)
  {
    error_log("mm_start()","EMS OK");
    r.r_ax = 0x4100;
    intr(EMM_INT,&r);
    ems_frame[0] = MK_FP(r.r_bx,0x0000);
    ems_frame[1] = MK_FP(r.r_bx,0x4000);
    ems_frame[2] = MK_FP(r.r_bx,0x8000);
    ems_frame[3] = MK_FP(r.r_bx,0xC000);

    r.r_ax = 0x4600;
    intr(EMM_INT,&r);
    ems_ver_lo = r.r_ax & 0x0F;
    ems_ver_hi = (r.r_ax >> 4) & 0x0F;
  };
};


void mm_end(void)
{
  struct REGPACK r;
  
  if(xms_inst)
  {
    error_log("mm_end()","Freeing XMS-Swaparea");
    xms_free_swaparea();
  };

  if(_osmajor > 3)
  {
    error_log("mm_end()","Resetting allocation strategy");
    r.r_ax = 0x5801;
    r.r_bx = dos_mem_alloc;
    intr(DOS_INT,&r);
  };
  
  if(_osmajor > 4)
  {
    error_log("mm_end()","Resetting UMB-link");
    r.r_ax = 0x5803;
    r.r_bx = dos_umb_link;
    intr(DOS_INT,&r);
  };  
};


unsigned dos_get_free_mem(void)
{
  struct REGPACK r;
  
  error_log("dos_get_free_mem()","Called");
  r.r_ax = 0x4800;
  r.r_bx = 0xFFFF;
  intr(DOS_INT,&r);
  
  return(r.r_bx);
};


unsigned dos_alloc(unsigned paras)
{
  struct REGPACK r;
  
  error_log("dos_alloc()","Called");
  r.r_ax = 0x4800;
  r.r_bx = paras;
  intr(DOS_INT,&r);
  
  if(!(r.r_flags&1)) return(r.r_ax); else return(0);
};


void dos_free(unsigned segment)
{
  struct REGPACK r;
  
  error_log("dos_free()","Called");
  r.r_ax = 0x4900;
  r.r_es = segment;
  intr(DOS_INT,&r);
};


void xms_alloc_swaparea()
{
  unsigned int hlp;

  error_log("xms_alloc_swaparea()","Called");
  if(xms_page_type!=NONE) return;
  if(!xms_inst) return;
 
  if(!is_os2())
  {
    if(xms_alloc_hma())
    {
      error_log("xms_alloc_swaparea()","HMA allocated");
      xms_page = MK_FP(0xFFFF,0);
      xms_page_type = HMA;
      return;
    };
  };

  hlp = xms_alloc_umb(PAGESIZE>>10);
  if(hlp)
  {
    error_log("xms_alloc_swaparea()","UMB allocated");
    xms_page = MK_FP(hlp,0);
    xms_page_type = UMB;
    return;
  };

  if(dos_get_free_mem() > PAGESIZE>>4)
  {
    error_log("xms_alloc_swaparea()","DOS-MEM allocated");
    hlp = dos_alloc(PAGESIZE>>4);
    xms_page = MK_FP(hlp,0);
    xms_page_type = RAM;
  };
};

void xms_free_swaparea()
{
  struct REGPACK r;
  
  error_log("xms_free_swaparea()","Called");
  switch(xms_page_type)
  {
    case RAM:
      dos_free(FP_SEG(xms_page));
      break;
    case HMA:
      xms_free_hma();
      break;
    case UMB:
      xms_free_umb(FP_SEG(xms_page));
      break;
  };
  error_log("xms_free_swaparea()","OK");
  xms_page_type = NONE;
};



unsigned int xms_get_free()
{
  error_log("xms_get_free()","Called");
  _AX = 0x0800;
  xms_service();
  return(_AX);
};


unsigned char xms_alloc_hma()
{
  error_log("xms_alloc_hma()","Called");
  _DX = 0xFFFF;
  _AX = 0x0100;
  xms_service();
  return(_AX);
};

void xms_free_hma()
{
  error_log("xms_free_hma()","Called");
  _AX = 0x0200;
  xms_service();
};

unsigned int xms_alloc_umb(unsigned int size)
{
  error_log("xms_alloc_umb()","Called");
  _DX = size;
  _AX = 0x1000;
  xms_service();
  if(_AX) return(_BX); else return(0);
};

void xms_free_umb(unsigned int segment)
{
  error_log("xms_free_umb()","Called");
  _BX = segment;
  _AX = 0x1100;
  xms_service();
};

unsigned int xms_alloc(unsigned int size)
{
  error_log("xms_alloc()","Called");
  _DX = size;
  _AX = 0x0900;
  xms_service();
  if(_AX) return(_DX); else return(0);
};

void xms_free(unsigned int handle)
{
  error_log("xms_free()","Called");
  _DX = handle;
  _AX = 0x0A00;
  xms_service();
};

void xms_moveto(void far *ptr, unsigned int handle, unsigned long offset, unsigned long size)
{
  error_log("xms_moveto()","Called");
  if(size&1) size++;
  block.size = size;
  block.src_handle = 0;
  block.src_ofs = (unsigned long)ptr;
  block.dest_handle = handle;
  block.dest_ofs = offset;

  asm {
    mov ax,0b00h
    mov si,offset block
    call dword ptr [offset xms_service]
  };
};

void xms_movefrom(unsigned int handle, unsigned long offset, void far *ptr, unsigned long size)
{
  error_log("xms_movefrom()","Called");
  if(size&1) size++;
  block.size = size;
  block.src_handle = handle;
  block.src_ofs = offset;
  block.dest_handle = 0;
  block.dest_ofs = (unsigned long)ptr;

  asm {
    mov ax,0b00h
    mov si,offset block
    call dword ptr [offset xms_service]
  };
};



unsigned ems_get_free_pages()
{
  union REGS r;

  error_log("ems_get_free_pages()","Called");
  r.h.ah = 0x42;
  int86(EMM_INT,&r,&r);
  return(r.x.bx);
};

unsigned ems_alloc(unsigned pages)
{
  union REGS r;

  error_log("ems_alloc()","Called");
  r.h.ah = 0x43;
  r.x.bx = pages;
  int86(EMM_INT,&r,&r);
  return(r.x.dx);
};

void ems_setmap(unsigned handle, unsigned logical, unsigned char physical)
{
  union REGS r;

  error_log("ems_setmap()","Called");
  r.h.ah = 0x44;
  r.h.al = physical;
  r.x.bx = logical;
  r.x.dx = handle;
  int86(EMM_INT,&r,&r);
  return;
};

void ems_free(unsigned handle)
{
  union REGS r;

  error_log("ems_free()","Called");
  r.h.ah = 0x45;
  r.x.dx = handle;
  int86(EMM_INT,&r,&r);
  return;
};


