// QSL.C
// Written by Stefan Pfeiffer, 950425
// Last Modification: 950429
// (C) 1995 by Stefan Pfeiffer, DL1ELY

#include "qsl.h"


struct qso_entry huge *qsos;
UINT qso_cnt;


unsigned main(int argc, char **argv)
{
   BYTE cnt_name[80];
   struct qso_filter filter;
   BYTE do_sort;
   PBYTE cp;
   struct print_params param;
   FILE *cntf;
   struct date_rec date;
   UINT cnt_qrg;
   BYTE hlp[20];
   struct header h;


   printf("%s %s by %s, %s\n\n",TITLE,VERSION,AUTHOR,DATE);
   printf(" Dieses Programm druckt QSL-Karten fr eine mit FiveNine geloggte CNT-Datei.\n\n");
   printf(" Testdruck mit \"QSL -TEST\"\n");
   printf(" Druckparameter einstellen mit \"QSL -SET\"\n");

   if(argc == 1)
   {
      printf("\n Kein Dateiname einer CNT-Datei angegeben!\n\n");
      return(0);
   };

   strcpy(cnt_name,argv[1]);
   strupr(cnt_name);
   if(cnt_name[0] == '-')
   {
      if(!strcmp(cnt_name,"-SET"))
      {
         query_printer_params(&param);
         write_printer_params(param);
         return(0);
      };
      if(!strcmp(cnt_name,"-TEST"))
      {
         if(!get_printer_params(&param))
         {
            query_printer_params(&param);
            write_printer_params(param);
         };
         display_print_params(param);
         convert_print_params(&param);
         test_print(param);
         return(0);
      };
   };

   cp = strchr(cnt_name,'.');
   if(!cp) strcat(cnt_name,".CNT");

   if((cntf = fopen(cnt_name,"rb")) == NULL)
   {
      printf(" Datei %s nicht gefunden!\n\n",cnt_name);
      return(0);
   };

   fread(&h,1,HEADER_SIZE,cntf);
   printf(" %s enthlt %u QSOs.\n\n",cnt_name,h.qso_cnt);

   get_qso_mask(&filter);

   printf(" Soll der Ausdruck alphabetisch sortiert erfolgen? (J/N) ");
   do 
     do_sort = toupper(getch());
   while(do_sort != 'J' && do_sort != 'N');
   printf("%c\n",do_sort);

   alloc_qsos(h.qso_cnt);
   if(!qsos) return(0);

   load_qsos(cntf,filter);
   if(do_sort == 'J') sort_qsos();

   printf(" %d QSOs entsprachen den Bedingungen.\n",qso_cnt);
   if(!qso_cnt) return(0);

   get_date(&date);

   cnt_qrg = 0;
   printf(" Frequenz des benutzten Bandes in MHz (144, 432, 1296...): ");
   do
   {
      gets(hlp);
      cnt_qrg = atoi(hlp);
   } while(!cnt_qrg);

   if(!get_printer_params(&param))
   {
      query_printer_params(&param);
      write_printer_params(param);
   };
   display_print_params(param);

   print_qsos(h.owncall,h.ownloc,cntf,date,cnt_qrg,param);

   fclose(cntf);

   free_qsos();

   return(0);
};

void get_date(struct date_rec *d)
{
   BYTE hlp[20];

   d->day = 0;
   printf(" Tag des Kontests (1..31): ");
   do
   {
      gets(hlp);
      d->day = atoi(hlp);
      if(d->day > 31) d->day = 0;
   } while(!d->day);

   d->mon = 0;
   printf(" Monat des Kontests (1..12): ");
   do
   {
      gets(hlp);
      d->mon = atoi(hlp);
      if(d->mon > 12) d->mon = 0;
   } while(!d->mon);

   d->year = 0;
   printf(" Jahr des Kontests (94..99): ");
   do
   {
      gets(hlp);
      d->year = atoi(hlp);
      if(d->year < 94) d->year = 0;
      if(d->year > 99) d->year = 0;
   } while(!d->year);
};

void get_qso_mask(struct qso_filter *filter)
{
   BYTE hlp[20];

   printf(" Zuerst kann eingeschrnkt werden, fr welche QSOs QSL-Karten gedruckt\n");
   printf(" werden sollen. Wird immer mit <CR> geantwortet, werden ALLE QSOs ausgedruckt.\n");
   printf(" Alle gemachten Angaben mssen auf ein QSO GLEICHZEITIG zutreffen, damit es\n");
   printf(" ausgedruckt wird! (UND-Verknpfung)\n");
   printf(" Callmaske (z.B. \"SM1*\" oder \"*0ABC\" wie mit DOS-Wildcards): ");
   gets(filter->call);
   strupr(filter->call);
   if(!filter->call[0]) strcpy(filter->call,"*");

   printf(" Locatormaske (z.B. \"JN49*\"): ");
   gets(filter->locator);
   strupr(filter->locator);
   if(!filter->locator[0]) strcpy(filter->locator,"*");

   printf(" Mindestentfernung in Kilometern (<CR> = 0km): ");
   filter->min_dist = 0xFFFF;
   do
   {
     gets(hlp);
     if(!hlp[0]) filter->min_dist = 0;
     else filter->min_dist = atoi(hlp);
   } while(filter->min_dist == 0xFFFF);
};


void query_printer_params(struct print_params *p)
{
   BYTE hlp[20];

   printf("\n Druckdaten:\n\n");

   p->unit = 180;
   printf(" Kleinster vertikaler Vorschub in 1/xx Inch (180 oder 216), <CR> = 180): ");
   gets(hlp);
   if(hlp[0]) p->unit = atoi(hlp);

   p->vorlauf = 0;
   printf(" Einmaliger Blattvorschub zu Beginn des Ausdrucks\n (in 1/10 Millimetern, <CR> = 0): ");
   gets(hlp);
   if(hlp[0]) p->vorlauf = atoi(hlp);

   p->etikett_rand = 0;
   printf(" Linker Blattrand (in 1/10 Millimetern, <CR> = 0): ");
   gets(hlp);
   if(hlp[0]) p->etikett_rand = atoi(hlp);

   p->linker_rand = 0;
   printf(" Abstand zwischen linkem Rand des Etiketts und Druckbereich\n (In 1/10 Millimetern, <CR> = 0): ");
   gets(hlp);
   if(hlp[0]) p->linker_rand = atoi(hlp);

   p->rechter_rand = 0;
   printf(" Abstand zwischen Druckbereich und rechtem Rand des Etiketts\n (In 1/10 Millimetern, <CR> = 0): ");
   gets(hlp);
   if(hlp[0]) p->rechter_rand = atoi(hlp);

   p->oberer_rand = 0;
   printf(" Abstand zwischen oberem Rand des Etiketts und Druckbereich\n (In 1/10 Millimetern, <CR> = 0): ");
   gets(hlp);
   if(hlp[0]) p->oberer_rand = atoi(hlp);

   p->unterer_rand = 0;
   printf(" Abstand zwischen Druckbereich und unterem Rand des Etiketts\n (In 1/10 Millimetern, <CR> = 0): ");
   gets(hlp);
   if(hlp[0]) p->unterer_rand = atoi(hlp);

   p->breite = 0;
   printf(" Breite des gesamten Etiketts (in 1/10 Millimetern): ");
   do
   {
      gets(hlp);
      if(hlp[0]) p->breite = atoi(hlp);
   } while(!p->breite);

   p->hoehe = 0;
   printf(" Hhe der gesamten Etiketts (in 1/10 Millimetern): ");
   do
   {
      gets(hlp);
      if(hlp[0]) p->hoehe = atoi(hlp);
   } while(!p->hoehe);

   p->abstand = 0;
   printf(" Vertikaler Abstand der Etiketten (in 1/10 Millimetern, <CR> = 0): ");
   gets(hlp);
   if(hlp[0]) p->abstand = atoi(hlp);
   printf("\n Nutzbare Flche (Breite x Hoehe): %2.1fmm x %2.1fmm\n",
          (float)(p->breite-p->linker_rand-p->rechter_rand)/10.0,
          (float)(p->hoehe-p->oberer_rand-p->unterer_rand)/10.0);
};

void write_printer_params(struct print_params p)
{
   FILE *prnfile;

   if((prnfile = fopen("PRINTER.59","wt")) == NULL) return;

   fprintf(prnfile,"%u\n",p.unit);
   fprintf(prnfile,"%u\n",p.vorlauf);
   fprintf(prnfile,"%u\n",p.etikett_rand);
   fprintf(prnfile,"%u\n",p.linker_rand);
   fprintf(prnfile,"%u\n",p.rechter_rand);
   fprintf(prnfile,"%u\n",p.oberer_rand);
   fprintf(prnfile,"%u\n",p.unterer_rand);
   fprintf(prnfile,"%u\n",p.breite);
   fprintf(prnfile,"%u\n",p.hoehe);
   fprintf(prnfile,"%u\n",p.abstand);

   fclose(prnfile);
};

#define CHARUNIT    (60/12)

void test_print(struct print_params p)
{
  UINT x_usable;
  UINT y_usable;

  x_usable = p.breite-p.linker_rand-p.rechter_rand;
  y_usable = p.hoehe-p.oberer_rand-p.unterer_rand;

  vertvor(p.vorlauf,p.unit);
  horzvor(p.etikett_rand);
  fprintf(stdprn,"%c",218);
  horzvor(p.etikett_rand+p.breite);
  fprintf(stdprn,"%c",191);

  vertvor(p.oberer_rand,p.unit);
  horzvor(p.etikett_rand+p.linker_rand);
  fprintf(stdprn,"%c",218);
  horzvor(p.etikett_rand+p.linker_rand+x_usable);
  fprintf(stdprn,"%c",191);

  vertvor(y_usable,p.unit);
  horzvor(p.etikett_rand+p.linker_rand);
  fprintf(stdprn,"%c",192);
  horzvor(p.etikett_rand+p.linker_rand+x_usable);
  fprintf(stdprn,"%c",217);

  vertvor(p.unterer_rand,p.unit);
  horzvor(p.etikett_rand);
  fprintf(stdprn,"%c",192);
  horzvor(p.etikett_rand+p.breite);
  fprintf(stdprn,"%c",217);

  vertvor(p.abstand,p.unit);
  horzvor(p.etikett_rand);
  fprintf(stdprn,"%c%c%c",196,196,196);
};



BYTE get_printer_params(struct print_params *p)
{
   FILE *prnfile;
   BYTE s[20];

   if((prnfile = fopen("PRINTER.59","rt")) == NULL) return(0);

   fgets(s,20,prnfile);
   p->unit = atoi(s);

   fgets(s,20,prnfile);
   p->vorlauf = atoi(s);

   fgets(s,20,prnfile);
   p->etikett_rand = atoi(s);

   fgets(s,20,prnfile);
   p->linker_rand = atoi(s);

   fgets(s,20,prnfile);
   p->rechter_rand = atoi(s);

   fgets(s,20,prnfile);
   p->oberer_rand = atoi(s);

   fgets(s,20,prnfile);
   p->unterer_rand = atoi(s);

   fgets(s,20,prnfile);
   p->breite = atoi(s);

   fgets(s,20,prnfile);
   p->hoehe = atoi(s);

   fgets(s,20,prnfile);
   p->abstand = atoi(s);

   fclose(prnfile);

   return(1);
};

unsigned char load_next_qso(FILE *f, struct qso *q)
{
  UINT read;
  
  memset((void *)q,0,QSO_SIZE);

  read = 0;
  read+=fread((void *)&q->hour,1,2,f);
  read+=fread((void *)&q->min,1,2,f);
  read+=fread((void *)&q->nr,1,2,f);
  read+=fread((void *)q->barecall,1,13,f);
  read+=fread((void *)q->call,1,14,f);
  read+=fread((void *)q->locator,1,7,f);
  read+=fread((void *)q->rst_RX,1,8,f);
  read+=fread((void *)q->rst_TX,1,4,f);
  read+=fread((void *)&q->points,1,2,f);
  read+=fread((void *)&q->direction,1,2,f);
  
  return(read==56);
};  


void alloc_qsos(UINT cnt)
{
   unsigned long bytes;
   UINT seg;

   bytes = cnt*sizeof(struct qso_entry);
   bytes = (bytes+15)>>4;

   if(allocmem(bytes,&seg) == -1)
   {
      qsos = MK_FP(seg,0);
      qso_cnt = 0;
   };
};

void load_qsos(FILE *f,struct qso_filter filter)
{
  struct qso q;
  unsigned long p;
  PBYTE av[20];
  BYTE i;

  p = ftell(f);
  while(load_next_qso(f,&q))
  {
     if(q.points>=filter.min_dist)
     {
        memset(&av,0,sizeof(av));
        if(!strcmp(filter.call,"*") || wildmat(q.call,filter.call,av))
        {
           for(i = 0;av[i] != (char *)0;++i) free(av[i]);
           memset(&av,0,sizeof(av));
           if(!strcmp(filter.locator,"*") || wildmat(q.locator,filter.locator,av))
           {
              strcpy(qsos[qso_cnt].call,q.call);
              qsos[qso_cnt].pos = p;
              qso_cnt++;
           };
        };
        for(i = 0;av[i] != (char *)0;++i) free(av[i]);
      };
      p = ftell(f);
   };
};


void quicksort(struct qso_entry huge *ptr,UINT first,UINT last,int (*fcmp)(const void *, const void *))
{
  UINT i;
  UINT j;
  UINT m;
  struct qso_entry p;

  i = first;
  j = last;
  m = (i+j)/2;

  while(i<j)
  {
    while((i<m) && (fcmp(ptr+i,ptr+m)<0)) i++;
    while((j>m) && (fcmp(ptr+m,ptr+j)<0)) j--;

    if(i!=j)
    {
      memcpy(&p,ptr+i,20);
      memcpy(ptr+i,ptr+j,20);
      memcpy(ptr+j,&p,20);
      if(m==i) m=j;
      else if(m==j) m=i;
      if(i<m) i++;
      if(j>m) j--;
    };
  };

  if(last-first>1)
  {
    if(i>first) quicksort(ptr,first,i,fcmp);
    if(j<last) quicksort(ptr,j,last,fcmp);
  };
};


int far cmp_index(const void *a,const void *b)
{
  return(strncmp(((struct qso_entry *)a)->call,((struct qso_entry *)b)->call,16));
};


void sort_qsos(void)
{
   quicksort(qsos,0,qso_cnt-1,cmp_index);
};


#define TO_RADIO    "To Radio: "
#define TEXT1       "I'm happy to cfm our contest-QSO:"
#define TEXT2       "  Date     UTC    Mode   MHz   QRB "
#define TEXT3       "%02d.%02d.%02d  %02d:%02d   %s    %d  %ukm"
#define TEXT4       "ur number: %s  my number: %s%s"
#define TEXT5       "pse QSL!"
#define TEXT6       "VY 73 de %s in %s"

void print(PBYTE oc,PBYTE ol,FILE *f,UINT nr,struct date_rec date,UINT qrg,struct print_params param)
{
   struct qso q;
   UINT hlp, hlp2;
   UINT rest;
   UINT needed;
   UINT unused;
   UINT x_usable;
   UINT y_usable;
   BYTE s[80];
   BYTE s1[10];
   BYTE s2[10];
   BYTE s3[5];
   BYTE *cp1;
   BYTE *cp2;

   x_usable = param.breite-param.linker_rand-param.rechter_rand;
   y_usable = param.hoehe-param.oberer_rand-param.unterer_rand;

   needed = param.unit == 216? 212 : 177;
   unused = (y_usable-needed)>>1;

   fseek(f,qsos[nr].pos,SEEK_SET);
   load_next_qso(f,&q);
   if(q.hour<14) date.day++;

   rest = y_usable;
   vertvor(param.oberer_rand,param.unit);

   hlp = (strlen(TO_RADIO)*CHARUNIT)+(strlen(q.call)*2*CHARUNIT);
   hlp2 = (x_usable-hlp)>>1;
   horzvor(param.etikett_rand+param.linker_rand+hlp2);
   fprintf(stdprn,TO_RADIO);

   fprintf(stdprn,"\x1BW%c",1);
   fprintf(stdprn,"%s",q.call);
   fprintf(stdprn,"\x1BW%c",0);
   vertvor(unused,param.unit);
   rest-=unused;


   hlp = strlen(TEXT1)*CHARUNIT;
   hlp2 = (x_usable-hlp)>>1;
   horzvor(param.etikett_rand+param.linker_rand+hlp2);
   fprintf(stdprn,TEXT1);
   vertvor(param.unit == 216 ? 54 : 45,param.unit);
   rest-=param.unit == 216 ? 54 : 45;

   hlp = strlen(TEXT2)*CHARUNIT;
   hlp2 = (x_usable-hlp)>>1;
   horzvor(param.etikett_rand+param.linker_rand+hlp2);
   fprintf(stdprn,TEXT2);
   vertvor(param.unit == 216 ? 32 : 27,param.unit);
   rest-=param.unit == 216 ? 32 : 27;
   strcpy(s3,"SSB");
   if(q.rst_TX[0] != ' ')
   {
      if(q.rst_TX[2] != 'A') strcpy(s3," CW");
      else strcpy(s3,"AUR");
   };

   sprintf(s,TEXT3,date.day,date.mon,date.year,q.hour,q.min,s3,qrg,q.points);
   horzvor(param.etikett_rand+param.linker_rand+hlp2);
   fprintf(stdprn,s);
   vertvor(param.unit == 216 ? 54 : 45,param.unit);
   rest-=param.unit == 216 ? 54 : 45;

   strcpy(s1,q.rst_RX);
   cp1 = s1;
   while(*cp1 == ' ') cp1++;
   strcpy(s2,q.rst_TX);
   cp2 = s2;
   while(*cp2 == ' ') cp2++;
   if(q.nr<1000) sprintf(s3,"%03d",q.nr);
   else sprintf(s3,"%04d",q.nr);
   sprintf(s,TEXT4,cp1,cp2,s3);
   hlp = strlen(s)*CHARUNIT;
   hlp2 = (x_usable-hlp)>>1;
   horzvor(param.etikett_rand+param.linker_rand+hlp2);
   fprintf(stdprn,s);
   vertvor(unused,param.unit);
   rest-=unused;

   horzvor(param.etikett_rand+param.linker_rand+(2*CHARUNIT));
   fprintf(stdprn,TEXT5);
   vertvor(param.unit == 216 ? 36 : 30,param.unit);
   sprintf(s,TEXT6,oc,ol);
   horzvor(param.etikett_rand+param.linker_rand+(2*CHARUNIT));
   fprintf(stdprn,s);
   vertvor(param.unit == 216 ? 36 : 30,param.unit);
   rest-=param.unit == 217 ? 72 : 60;
   vertvor(rest,param.unit);
   vertvor(param.unterer_rand,param.unit);
};

void display_print_params(struct print_params p)
{
   printf("\nRnder: rechts %2.1fmm, links %2.1fmm, oben %2.1fmm, unten %2.1fmm\n",p.rechter_rand/10.0,p.linker_rand/10.0,p.oberer_rand/10.0,p.unterer_rand/10.0);
   printf("linker Blattrand: %2.1fmm, Etikette: %2.1fmm x %2.1fmm, Druckflche %2.1fmm x %2.1fmm\n",p.etikett_rand/10.0,
          p.breite/10.0,p.hoehe/10.0,(p.breite-p.linker_rand-p.rechter_rand)/10.0,(p.hoehe-p.oberer_rand-p.unterer_rand)/10.0);
   printf("Etikettenabstand: %2.1fmm, kleinster vertikaler Druckervorschub: 1/%u Inch\n",p.abstand/10.0,p.unit);
};

void convert_print_params(struct print_params *p)
{
   p->vorlauf = (UINT)((float)p->vorlauf * MM2INCH * (float)p->unit);
   p->etikett_rand = (UINT)((float)p->etikett_rand * MM2INCH *(float)60);
   p->linker_rand = (UINT)((float)p->linker_rand * MM2INCH * (float)60);
   p->rechter_rand = (UINT)((float)p->rechter_rand * MM2INCH * (float)60);
   p->oberer_rand = (UINT)((float)p->oberer_rand * MM2INCH *(float)p->unit);
   p->unterer_rand = (UINT)((float)p->unterer_rand * MM2INCH * (float)p->unit);
   p->breite = (UINT)((float)p->breite * MM2INCH * (float)60);
   p->hoehe = (UINT)((float)p->hoehe * MM2INCH * (float)p->unit);
   p->abstand = (UINT)((float)p->abstand * MM2INCH * (float)p->unit);
};

void print_qsos(PBYTE oc,PBYTE ol,FILE *f,struct date_rec date,UINT qrg,struct print_params param)
{
   UINT i;

   convert_print_params(&param);

   fprintf(stdprn,"\x1Bx1");
   fprintf(stdprn,"\x1B\x7E%c",1);
   fprintf(stdprn,"\x1B\x4D");
   fprintf(stdprn,"\x1B\x70%c",0);

   vertvor(param.vorlauf,param.unit);
   for(i=0;i<qso_cnt;i++)
   {
      print(oc,ol,f,i,date,qrg,param);
      vertvor(param.abstand,param.unit);
   };

   fprintf(stdprn,"\x1B\x7E%c",0);
   fprintf(stdprn,"\xFF");
};


void free_qsos(void)
{
   freemem(FP_SEG(qsos));
   qso_cnt = 0;
   qsos = NULL;
};

// Vertikaler Vorschub, Value/216stel oder Value/180stel Inch
void vertvor(UINT value, BYTE max)
{
   while(value)
   {
      fprintf(stdprn,"\x1BJ%c",value>max?max:value);
      value-=value>max?max:value;
   };
};

// Horizontaler Vorschub, Value/60stel Inch
void horzvor(UINT value)
{
  fprintf(stdprn,"\x1B$%c%c",value%256,value>>8);
};

