// SIO.C - Serial Input/Output
// Written by Stefan Pfeiffer, DL1ELY, 950714
// Last modification: 950824
// (C) 1995 by Stefan Pfeiffer, DL1ELY@DB0PKE

#include <dos.h>
#include <conio.h>
#include "secmem.h"
#include "sio.h"

unsigned char int_table[16] = {0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
                               0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77};

unsigned def_base[5]     = {0x000,0x3F8,0x2F8,0x3E8,0x2E8};
unsigned char def_irq[5] = {0,4,3,4,3};

#include "sio_ints.c"

struct port_des *com[5];


void sio_init_port(struct port_des *p)
{
  unsigned char status = 0x03;  // 8 Bit, 1 Stopbit, kein Paritt
  unsigned int divisor;
  unsigned char dummy;

  if(!p->rx_size) p->rx_size = 4096;
  p->rx_buf = sec_alloc(p->rx_size);
  p->rx_write = 0;
  p->rx_read = 0;
  p->rx_sum = 0;

  if(!p->tx_size) p->tx_size = 4096;
  p->tx_buf = sec_alloc(p->tx_size);
  p->tx_write = 0;
  p->tx_read = 0;
  p->tx_sum = 0;

  p->error = 0;

  if(!p->irq) p->irq = def_irq[p->com];
  if(!p->base) p->base = def_base[p->com];

  // Interrupt setzen
  p->old_handler = getvect(int_table[p->irq]);

  switch(p->com)
  {
     case 1:
        setvect(int_table[p->irq],sio_int1);
        break;
     case 2:
        setvect(int_table[p->irq],sio_int2);
        break;
     case 3:
        setvect(int_table[p->irq],sio_int3);
        break;
     case 4:
        setvect(int_table[p->irq],sio_int4);
        break;
  };
 

  divisor = (MAX_BAUD/p->baudrate);

  // Port-Status setzen, Baudratenprogrammierung aktivieren
  outp(p->base+OFS_LCR,status|0x80);

  // Baudrate programmieren
  outp(p->base+OFS_TX,divisor&0xFF);
  outp(p->base+OFS_IER,divisor>>8);

  // Baudratenprogrammierung beenden
  outp(p->base+OFS_LCR,status);

  // 16550er programmieren
  outp(p->base+OFS_FCR,0x87);

  p->tx_empty = 1;

  // Interrupt-Aktivierung setzen
  outp(p->base+OFS_IER,0x07);

  // Interruptfreigabe
  outp(p->base+OFS_MCR,0x08);

  asm cli;

  dummy = inp((p->irq<8?PORT8259_1:PORT8259_2)+OFS_OCW1);
  dummy &= (0xFF^(1<<(p->irq&0x07)));

  outp((p->irq<8?PORT8259_1:PORT8259_2)+OFS_OCW1,dummy);

  asm sti;

  p->used = COM_USED;
};


void sio_close_port(struct port_des *p)
{
   unsigned char dummy;

   asm cli;

   dummy = inp((p->irq<8?PORT8259_1:PORT8259_2)+OFS_OCW1);
   dummy |= (1<<(p->irq&0x07));
   outp((p->irq<8?PORT8259_1:PORT8259_2)+OFS_OCW1,dummy);

   asm sti;

   setvect(int_table[p->irq],p->old_handler);
   outp(p->base+OFS_IER,0);
   outp(p->base+OFS_MCR,0);

   sec_free(p->rx_buf);
   sec_free(p->tx_buf);

   p->used = 0;
};


void sio_tx(struct port_des *p, unsigned int cnt, char *buf)
{
   asm cli;

   while(cnt)
   {
      *(p->tx_buf+p->tx_write) = *buf;
      buf++;
      p->tx_write++;
      if(p->tx_write == p->tx_size) p->tx_write = 0;
      if(p->tx_write == p->tx_read) p->error++;
      cnt--;
   };

   if(p->tx_empty)
   {
      p->tx_empty = 0;
      outp(p->base+OFS_TX,*(p->tx_buf+p->tx_read));
      p->tx_read++;
      if(p->tx_read == p->tx_size) p->tx_read = 0;
      p->tx_sum++;
   };

   asm sti;
};

void sio_rx(struct port_des *p, unsigned int cnt, char *buf)
{
   while(cnt)
   {
      while(p->rx_read == p->rx_write) ;
      *buf = *(p->rx_buf+p->rx_read);
      p->rx_read++;
      if(p->rx_read == p->rx_size) p->rx_read = 0;
      cnt--;
   };
};

unsigned sio_rx_bytes(struct port_des *p)
{
   if(p->rx_read<=p->rx_write) return(p->rx_write-p->rx_read);
   else return(p->rx_size-p->rx_read+p->rx_write);
};

